"""
These are the django settings for the Seattle Clearinghouse project. See the README.txt
file for details on what needs to be set in this file. At a minimum for
development, it will be the database connection info and the SECRET_KEY value.

For public deployment, see the README.txt file for information about which
additional changes you'll need to make to this file.
"""

import os


from seattlegeni.common.util import log



# If DEBUG is True, then error details will be shown on the website and ADMINS
# will not receive an email when an error occurs. So, this should be False in
# production.
DEBUG = True
TEMPLATE_DEBUG = DEBUG

# The log level used by the seattlegeni log module. All messages at this level
# or more severe will be logged.
SEATTLECLEARINGHOUSE_LOG_LEVEL = log.LOG_LEVEL_DEBUG

# Rather than make the log module have to import this settings file to set the
# log level, just set it right here.
log.set_log_level(SEATTLECLEARINGHOUSE_LOG_LEVEL)

# This is needed to allow xmlrpc requests to work when they don't have a slash
# on the end of the url.
APPEND_SLASH = False

# The directory the settings.py file is in is what we consider the root of the website. 
SEATTLECLEARINGHOUSE_WEBSITE_ROOT = os.path.dirname(__file__)

# The directory where we keep the public keys of the node state keys.
SEATTLECLEARINGHOUSE_STATE_KEYS_DIR = os.path.join(SEATTLECLEARINGHOUSE_WEBSITE_ROOT, '..', 'node_state_transitions', 'statekeys')

# The XML-RPC interface to the Custom Installer Builder.
SEATTLECLEARINGHOUSE_INSTALLER_BUILDER_XMLRPC = "https://custombuilder.poly.edu/custom_install/xmlrpc/"

# Not currently used. This is left in for legacy installs
# The directory where the base installers named seattle_linux.tgz, seattle_mac.tgz,
# and seattle_win.zip are located.
#SEATTLECLEARINGHOUSE_BASE_INSTALLERS_DIR = "/var/www/dist"

# Not currently used. This is left in for legacy installs
# The directory in which customized installers created by seattlegeni will be
# stored. A directory within this directory will be created for each user.
#SEATTLECLEARINGHOUSE_USER_INSTALLERS_DIR = os.path.join(SEATTLECLEARINGHOUSE_BASE_INSTALLERS_DIR, "geni")

# Not currently used. This is left in for legacy installs
#The url that corresponds to SEATTLECLEARINGHOUSE_USER_INSTALLERS_DIR
#SEATTLECLEARINGHOUSE_USER_INSTALLERS_URL = "https://blackbox.cs.washington.edu/dist/geni"

# Need to specify the LOGIN_URL, as our login page isn't at the default login
# location (the default is /accounts/login).
LOGIN_URL = 'login'
# Users will be redirected to SOCIAL_AUTH_LOGIN_ERROR_URL in case of backend error/user cancellation
# during login or association (account linking).
SOCIAL_AUTH_LOGIN_ERROR_URL ='error'
SOCIAL_AUTH_BACKEND_ERROR_URL = 'error'
# When a user logs in with OpenID/OAuth send them to the profile page
SOCIAL_AUTH_LOGIN_REDIRECT_URL = 'profile'
# A new user created through OpenID/OAuth gets sent here upon creation. 
# currently not used
#SOCIAL_AUTH_NEW_USER_REDIRECT_URL = 'new_auto_register_user'

# fields specificed here will not be automatically changed/updated by social_auth
SOCIAL_AUTH_PROTECTED_USER_FIELDS = ['email',]

# Email addresses of people that should be emailed when a 500 error occurs on
# the site when DEBUG = False (that is, in production). Leave this to be empty
# if nobody should receive an email. 
ADMINS = (
    # ('Your Name', 'your_email@domain.com'),
)

# To be able to send mail to ADMINS when there is an error, django needs to
# know about an SMTP server it can use. That info is defined here.
#EMAIL_HOST = 'smtp.gmail.com'
#EMAIL_HOST_USER = 'an.error.sending.account@gmail.com'
#EMAIL_HOST_PASSWORD = 'PASSWORD_HERE'
#EMAIL_PORT = 587
#EMAIL_USE_TLS = True

# Email address that error notifications will be sent from.
#SERVER_EMAIL = "error@seattlegeni.server.hostname"

# We use this so we know which server the email came from by the subject line.
#EMAIL_SUBJECT_PREFIX = "[localhost] "

MANAGERS = ADMINS

DATABASE_ENGINE = 'mysql'      # 'postgresql_psycopg2', 'postgresql', 'mysql', 'sqlite3' or 'oracle'.
DATABASE_NAME = 'FILL_THIS_IN' # Or path to database file if using sqlite3.
DATABASE_USER = 'FILL_THIS_IN' # Not used with sqlite3.
DATABASE_PASSWORD = 'FILL_THIS_IN' # Not used with sqlite3.
DATABASE_HOST = ''             # Set to empty string for localhost. Not used with sqlite3.
DATABASE_PORT = ''             # Set to empty string for default. Not used with sqlite3.

if DATABASE_ENGINE == 'mysql':
  DATABASE_OPTIONS = {'init_command': 'SET storage_engine=INNODB'}

# Make this unique, and don't share it with anybody.
# Fill this in!
SECRET_KEY = ''

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# If running in a Windows environment this must be set to the same as your
# system time zone.
TIME_ZONE = 'America/Los_Angeles'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-us'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# Absolute path to the directory that holds media.
# Example: "/home/media/media.lawrence.com/"
MEDIA_ROOT = SEATTLECLEARINGHOUSE_WEBSITE_ROOT + '/html/media/'

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash if there is a path component (optional in other cases).
# Examples: "http://media.lawrence.com", "http://example.com/media/"
MEDIA_URL = '/site_media/'

# URL prefix for admin media -- CSS, JavaScript and images. Make sure to use a
# trailing slash.
# Examples: "http://foo.com/media/", "/media/".
ADMIN_MEDIA_PREFIX = '/admin_media/'

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
  'django.template.loaders.filesystem.load_template_source',
  'django.template.loaders.app_directories.load_template_source',
# 'django.template.loaders.eggs.load_template_source',
)

MIDDLEWARE_CLASSES = (
  'django.middleware.common.CommonMiddleware',
  'django.contrib.csrf.middleware.CsrfViewMiddleware',
  'django.contrib.csrf.middleware.CsrfResponseMiddleware',
  'django.contrib.sessions.middleware.SessionMiddleware',
  'django.contrib.auth.middleware.AuthenticationMiddleware',
  'django.contrib.messages.middleware.MessageMiddleware',
  #'django.middleware.doc.XViewMiddleware',

  # Our own middleware that logs when a request is initially received and
  # sets up the logger to log other messages with per-request unique ids.
  'seattlegeni.website.middleware.logrequest.LogRequestMiddleware',
  # Our own middleware that logs when unhandled exceptions happen.
  'seattlegeni.website.middleware.logexception.LogExceptionMiddleware',
)

ROOT_URLCONF = 'seattlegeni.website.urls'

TEMPLATE_DIRS = (
  # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
  # Always use forward slashes, even on Windows.
  # Don't forget to use absolute paths, not relative paths.
  SEATTLECLEARINGHOUSE_WEBSITE_ROOT + '/html/templates'
)

INSTALLED_APPS = (
  'django.contrib.admin',
  'django.contrib.auth',
  'django.contrib.contenttypes',
  'django.contrib.csrf',
  'django.contrib.sessions',
  'django.contrib.sites',
  'django.contrib.messages',
  # Needed for OpenID/OAuth login so must be listed. 
  'social_auth',

  # We have our maindb model defined here, so it must be listed.
  'seattlegeni.website.control',
)
  # Seattle Clearinghouse uses a django plugin called "django social auth" to handle
  # OpenID and OAuth.  The desired OpenID/OAuth providers must be listed here 
  # in order to be used.  Seattle Clearinghouse uses Facebook, Github, Windows Live
  # Google and Yahoo.  For more info visit https://github.com/omab/django-social-auth       
  # Google and Yahoo work without keys so they are enabled by default
AUTHENTICATION_BACKENDS = (
  
  #'social_auth.backends.facebook.FacebookBackend',
  'social_auth.backends.google.GoogleBackend',
  'social_auth.backends.yahoo.YahooBackend',
  #'social_auth.backends.contrib.github.GithubBackend',
  #'social_auth.backends.OpenIDBackend',
  #'social_auth.backends.browserid.BrowserIDBackend',
  #'social_auth.backends.contrib.live.LiveBackend',
  # Django default this is always needed and must always be last.
  'django.contrib.auth.backends.ModelBackend',	    
)
# Social_Auth needs OAuth keys in order to function.  Each backend provider has its
# own method to acquire keys usually requireing registering the web app with them.
# Yahoo and Google are OpenID and as such we do not need OAuth keys for them

#FACEBOOK_APP_ID                   = ''
#FACEBOOK_API_SECRET               = ''
#LIVE_CLIENT_ID                    = ''
#LIVE_CLIENT_SECRET                = ''
SOCIAL_AUTH_ERROR_KEY             = 'socialauth_error'
#GITHUB_APP_ID                     = ''
#GITHUB_API_SECRET                 = ''
# define what extra facebook permissions you would like from a user
#FACEBOOK_EXTENDED_PERMISSIONS = ['email']

TEMPLATE_CONTEXT_PROCESSORS = (
  'django.contrib.auth.context_processors.auth',
  'django.core.context_processors.debug',
  'django.core.context_processors.i18n',
  'django.core.context_processors.media',
  'django.contrib.messages.context_processors.messages',
  # Adds a social_auth dict with keys: are associated, not_associated and backends.
  # associated key is a list of UserSocialAuth instances associated with current user.
  # not_associated is a list of providers names that the current user doesn't have any association yet.
  # backends holds the list of backend names supported.  Each value is grouped by backend type openid, oauth2 and oauth
  'social_auth.context_processors.social_auth_by_type_backends',
)
#  Social_auth follows each of these in order and passes along a object with
#  information gathered to each function.
#  Custom fns can be written and passed in here we define them in seattlegeni.website.pipeline.
#  To use a custom fn you must call .save_status_to_session before your custom fn.
SOCIAL_AUTH_PIPELINE = (
  'seattlegeni.website.pipeline.custom_social_auth_user', 
  #'social_auth.backends.pipeline.associate.associate_by_email', 
  'social_auth.backends.pipeline.misc.save_status_to_session',
  'seattlegeni.website.pipeline.redirect_to_auto_register',
  'seattlegeni.website.pipeline.username',
  'seattlegeni.website.pipeline.custom_create_user',
  'social_auth.backends.pipeline.social.associate_user',
  'social_auth.backends.pipeline.social.load_extra_data',
  'social_auth.backends.pipeline.user.update_user_details',
)

# This is important for the partial pipeline, anytime it is broken with a custom
# function, it must be redirected back to SOCIAL_AUTH_COMPLETE_URL_NAME in order
# for the pipeline to continue.
SOCIAL_AUTH_COMPLETE_URL_NAME  = 'socialauth_complete'
#SOCIAL_AUTH_PROCESS_EXCEPTIONS = 'social_auth.utils.process_exceptions'

# DEBUG should be false for production. Useful for debugging social_auth problems.
SOCIAL_AUTH_RAISE_EXCEPTIONS = DEBUG
SOCIAL_AUTH_LAST_LOGIN = 'social_auth_last_login_backend'
#SOCIAL_AUTH_REDIRECT_IS_HTTPS = True

# The number of seconds sessions are valid for. Django uses this for the
# session expiration in the database in addition to the cookie expiration,
# which is good.
SESSION_COOKIE_AGE = 3600

# Use session cookies, not persistent cookies.
SESSION_EXPIRE_AT_BROWSER_CLOSE = True



"""
<Program>
  nodestatus.py

<Started>
  16 August 2009

<Author>
  Justin Samuel

<Purpose>
  This module provides a function, check_node(), which can be used to compare
  the information we have about a node in the database with the actual data
  the node reports. Optionally, the check_node() function will mark nodes as
  broken and release/mark for cleanup any vessels that are in an invalid
  state.
  
  Note that this module may be useful on the admin side of the website just
  for allowing an administrator to see what's wrong with a node. It's
  worth noting that this module isn't thread-safe, so if two requests made
  by administrators ran at the same time, the reported results they get back
  might be incorrect (especially if they are querying the same node at the same
  time). This lack of thread safety won't cause inaccurate actions to be taken
  on the database, though, because locks are used when not in readonly mode.
  
  A simple and lazy solution if lack of thread-safety becomes annoying would
  be to put the global data that is collected in a threading.local() data store
  and so not have to change much of the code. You might think that you could
  just require locks to be held for readonly requests, but that only reduces
  the problem, it doesn't eliminate it.

  TODO: check vessel ports
"""

import os

from seattlegeni.common.api import lockserver
from seattlegeni.common.api import maindb
from seattlegeni.common.api import nodemanager

from seattlegeni.common.util import log as log_module

from seattlegeni.common.exceptions import *

from seattlegeni.website import settings

from seattle.repyportability import *
add_dy_support(locals())

dy_import_module_symbols("rsa.r2py")


# Restore the log module as repy's log function is copied into our
# namespace when we are importing rsa.r2py
log = log_module




def _readfilecontents(filename):
  f = open(filename)
  contents = f.read()
  f.close()
  return contents





# Decrease the amount of logging output.
log.loglevel = log.LOG_LEVEL_INFO

# A dictionary where the names are names we use to refer to the keys and the
# values are the names of the public key files. All public key files will be
# considered relative to the directory specified in 
# settings.SEATTLECLEARINGHOUSE_STATE_KEYS_DIR.
statekeyfiles = {"acceptdonation" : "acceptdonation.publickey",
                 "canonical" : "canonical.publickey",
                 "onepercentmanyevents" : "onepercentmanyevents.publickey",
                 "movingto_onepercentmanyevents" : "movingto_onepercentmanyevents.publickey",
                 "twopercent" : "twopercent.publickey",
                 "movingto_twopercent" : "movingto_twopercent.publickey",
                 "movingto_canonical" : "movingto_canonical.publickey"
                 }

statekeys = {}
for keyname in statekeyfiles:
  keyfilefullpath = os.path.join(settings.SEATTLECLEARINGHOUSE_STATE_KEYS_DIR, statekeyfiles[keyname])
  statekeys[keyname] = rsa_string_to_publickey(_readfilecontents(keyfilefullpath))





# We'll keep track which nodes had problems.
nodes_with_problems = {}

# This holds a list of nodes/vessels where action either was taken or would
# have been taken if checknode was called with readonly set to False.
actionstaken = {"recorded_communication_failure":[],
                "node_marked_inactive":[],
                "node_marked_broken":[],
                "vessel_released":[]}





def _report_node_problem(node, message):
  global nodes_with_problems
  
  if not node.node_identifier in nodes_with_problems:
    nodes_with_problems[node.node_identifier] = []
  
  nodes_with_problems[node.node_identifier].append(message)
  
  log.info("Problem on node " + str(node) + ": " + message)





def get_node_problem_info():
  """
  <Purpose>
    Get the information about node problems collected from previous calls to
    check_node().
  <Arguments>
    None
  <Exceptions>
    None
  <Side Effects>
    None
  <Returns>
    A dictionary whose keys are the node identifiers and the values are lists
    of strings that describe the problems encountered.
  """
  return nodes_with_problems





def get_actions_taken():
  """
  <Purpose>
    Get the information about what database actions were taken for nodes and
    vessels.
  <Arguments>
    None
  <Exceptions>
    None
  <Side Effects>
    None
  <Returns>
    A dictionary whose keys are the the names of actions taken and the values
    are lists of nodes or vessels the actions were taken on.
  """
  return actionstaken





def reset_collected_data():
  """
  <Purpose>
    Reset the node problem info and actions taken that is collected.
  <Arguments>
    None
  <Exceptions>
    None
  <Side Effects>
    All previously collected node problem info has been discarded. The list of
    actions taken is also reset.
  <Returns>
    None
  """
  global nodes_with_problems
  nodes_with_problems = {}
  
  for action in actionstaken:
    actionstaken[action] = []





def _record_node_communication_failure(readonly, node):
  if node not in actionstaken["recorded_communication_failure"]:
    actionstaken["recorded_communication_failure"].append(node)
    
  if readonly:
    log.info(str(node) + " Not recording communication failure because called in readonly mode.")
  else:
    log.info(str(node) + " Recording communication failure.")
    maindb.record_node_communication_failure(node)





def _mark_node_inactive(readonly, node):
  if node not in actionstaken["node_marked_inactive"]:
    actionstaken["node_marked_inactive"].append(node)
    
  if readonly:
    log.info(str(node) + " Not marking node as inactive because called in readonly mode.")
  else:
    log.info(str(node) + " Marking node as inactive.")
    maindb.mark_node_as_inactive(node)





def _mark_node_broken(readonly, node):
  if node not in actionstaken["node_marked_broken"]:
    actionstaken["node_marked_broken"].append(node)
  
  if node.is_broken:
    log.info(str(node) + " Not marking node as broken because it is already broken.")
  else:
    if readonly:
      log.info(str(node) + " Not marking node as broken because called in readonly mode.")
    else:
      log.info(str(node) + " Marking node as broken.")
      maindb.mark_node_as_broken(node)





def _release_vessel(readonly, vessel):
  if vessel not in actionstaken["vessel_released"]:
    actionstaken["vessel_released"].append(vessel)
  
  if readonly:
    log.info(str(vessel) + " Not recording vessel as released because called in readonly mode.")
  else:
    log.info(str(vessel) + " Recording vessel as released.")
    maindb.record_released_vessel(vessel)




    
def check_node(node, readonly=True, lockserver_handle=None):
  """
  <Purpose>
    Check a node for problems. This will try to contact the node and will
    compare the information retrieved from the node to the information we
    have in our database. It will log and collect the information about
    the problems. The problem information can be retrieved program
  <Arguments>
    node
      The Node object of the node to be checked.
    readonly
      False if the function should mark the node in the database as inactive
      or broken (and vessels released) when appropriate, True if it should
      never change anything in the database. Default is True.
    lockserver_handle
      If an existing lockserver handle should be used for lock acquisitions,
      it should be provided here. Otherwise, a new lockserver handle will
      be used the during of this function call.
      Note: no locking is done if readonly is True. That is, if there is
      no reason to lock a node, there is no reason to provide a
      lockserver_handle.
  <Exceptions>
    None
  <Side Effects>
    If readonly is False, the database may be updated appropriately based on
    what the function sees. No changes are ever directly made to the nodes
    through nodemanager communication regardless of the setting of readonly.
    However, other scripts might take action based on database changes (e.g.
    released vessel will quickly be cleaned up by the backend daemon).
  <Returns>
    None
  """
    
  if not readonly:
    must_destroy_lockserver_handle = False
    
    if lockserver_handle is None:
      must_destroy_lockserver_handle = True
      lockserver_handle = lockserver.create_lockserver_handle()
      
    if not readonly:
      lockserver.lock_node(lockserver_handle, node.node_identifier)
    
  # Be sure to release the node lock, if we are locking the node.
  try:
    # Get a fresh node record from the database. It might have changed before
    # we obtained the lock.
    node = maindb.get_node(node.node_identifier)
    
    # The code beyond this point would be a good candidate for splitting out
    # into a few smaller functions for readability.
    
    donation_list = maindb.get_donations_from_node(node)
    if len(donation_list) == 0:
      _report_node_problem(node, "The node has no corresponding donation records. " +
                           "Not marking node broken, though.")
    
    try:
      nodeinfo = nodemanager.get_node_info(node.last_known_ip, node.last_known_port)
    except NodemanagerCommunicationError:
      _record_node_communication_failure(readonly, node)
      _report_node_problem(node, "Can't communicate with node.")
      return
    
    try:
      nodekey_str = rsa_publickey_to_string(nodeinfo["nodekey"])
    except ValueError:
      _mark_node_broken(readonly, node)
      _report_node_problem(node, "Invalid nodekey: " + str(nodeinfo["nodekey"]))
      return
    
    # Check that the nodeid matches. If it doesn't, it probably means seattle
    # was reinstalled or there is a different system at that address now.
    if node.node_identifier != nodekey_str:
      _mark_node_inactive(readonly, node)
      _report_node_problem(node, "Wrong node identifier, the node reports: " + str(nodeinfo["nodekey"]))
      # Not much more worth checking in this case.
      return
    
    
    # Check that the database thinks it knows the extra vessel name.
    if node.extra_vessel_name == "":
      _mark_node_broken(readonly, node)
      _report_node_problem(node, "No extra_vessel_name in the database.")
      # Not much more worth checking in this case.
      return
    
    # Check that a vessel by the name of extra_vessel_name exists on the node.
    if node.extra_vessel_name not in nodeinfo["vessels"]:
      _mark_node_broken(readonly, node)
      _report_node_problem(node, "The extra_vessel_name in the database is a vessel name that doesn't exist on the node.")
      # Not much more worth checking in this case.
      return
    
    extravesselinfo = nodeinfo["vessels"][node.extra_vessel_name]
        
    vessels_in_db = maindb.get_vessels_on_node(node)
  
    if len(extravesselinfo["userkeys"]) != 1:
      _mark_node_broken(readonly, node)
      _report_node_problem(node, "The extra vessel '" + node.extra_vessel_name + 
                          "' doesn't have 1 user key, it has " + 
                          str(len(extravesselinfo["userkeys"])))
  
    else:    
      # Figure out which state the node is in according to the state key.
      recognized_state_name = ""
    
      for statename in statekeys:
        if statekeys[statename] == extravesselinfo["userkeys"][0]:
          recognized_state_name = statename
    
      if not recognized_state_name:
        _mark_node_broken(readonly, node)
        _report_node_problem(node, "The extra vessel '" + node.extra_vessel_name + 
                            "' doesn't have a recognized user/state key")
    
      if len(vessels_in_db) == 0:
        if recognized_state_name == "onepercentmanyevents" or recognized_state_name == "twopercent":
          # We don't mark it as broken because it may be in transition by a
          # transition script away from onepercentmanyevents. If the vessels
          # in the db have been deleted first but the state key hasn't been
          # changed yet, we might hit this. Also, it's not so bad to have it
          # not be marked as broken when it's like this, as it has no vessels
          # we know about, anyways, so we're not going to be giving questionable
          # resources to users because of it.
          _report_node_problem(node, "The node is in the " + recognized_state_name + " state " + 
                              "but we don't have any vessels for it in the database.")
      else:
        if recognized_state_name != "onepercentmanyevents" and recognized_state_name != "twopercent":
          # We don't mark it as broken because it may be in transition by a
          # transition script. Also, we may have other states in the future
          # besides onepercentmanyevents that have vessels. We don't want
          # to make all of those nodes inactive if it's just an issue of
          # someone forgot to update this script.
          _report_node_problem(node, "The node is in the '" + recognized_state_name + 
                              "' state but we have vessels for it in the database.")
      
    known_vessel_names = []
    for vessel in vessels_in_db:
      known_vessel_names.append(vessel.name)
  
    # Look for vessels on the node with our node ownerkey which aren't in our database.
    for actualvesselname in nodeinfo["vessels"]:
  
      vessel_ownerkey = nodeinfo["vessels"][actualvesselname]["ownerkey"]
      
      try:
        vessel_ownerkey_str = rsa_publickey_to_string(vessel_ownerkey)
      except ValueError:
        # At this point we aren't sure it's our node, but let's assume that if
        # there's an invalid key then the node is broken, period.
        _mark_node_broken(readonly, node)
        _report_node_problem(node, "Invalid vessel ownerkey: " + str(vessel_ownerkey))
        return
      
      if vessel_ownerkey_str == node.owner_pubkey:
        if actualvesselname not in known_vessel_names and actualvesselname != node.extra_vessel_name:
          _mark_node_broken(readonly, node)
          _report_node_problem(node, "The vessel '" + actualvesselname + "' exists on the node " + 
                              "with the ownerkey for the node, but it's not in our vessels table.")
  
    # Do some checking on each vessel we have in our database.
    for vessel in vessels_in_db:
      
      # Check that the vessel in our database actually exists on the node.
      if vessel.name not in nodeinfo["vessels"]:
        _mark_node_broken(readonly, node)
        _report_node_problem(node, "The vessel '" + vessel.name + "' in our db doesn't exist on the node.")
        continue
  
      vesselinfo = nodeinfo["vessels"][vessel.name]
  
      try:
        vessel_ownerkey_str = rsa_publickey_to_string(vesselinfo["ownerkey"])
      except ValueError:
        _mark_node_broken(readonly, node)
        _report_node_problem(node, "Invalid vessel ownerkey on a vessel in our db: " + str(vessel_ownerkey))
        return
  
      # Check that the owner key for the vessel is what we have for the node's owner key in our database.
      if node.owner_pubkey != vessel_ownerkey_str:
        _mark_node_broken(readonly, node)
        _report_node_problem(node, "The vessel '" + vessel.name + "' doesn't have the ownerkey we use for the node.")
      
      if not vesselinfo["advertise"]:
        _mark_node_broken(readonly, node)
        _report_node_problem(node, "The vessel '" + vessel.name + "' isn't advertising.")
      
      # We're only concerned with non-dirty vessels as the backend daemon
      # should be working on cleaning up dirty vessels.
      if not vessel.is_dirty:
        # Check that the user keys that have access are the ones that should have access.
        users_with_access = maindb.get_users_with_access_to_vessel(vessel)
        
        if len(users_with_access) != len(vesselinfo["userkeys"]):
          _release_vessel(readonly, vessel)
          _report_node_problem(node, "The vessel '" + vessel.name + "' reports " + 
                              str(len(vesselinfo["userkeys"])) + " user keys, but we expected " + str(len(users_with_access)))
          
        for user in users_with_access:
          if rsa_string_to_publickey(user.user_pubkey) not in vesselinfo["userkeys"]:
            _release_vessel(readonly, vessel)
            _report_node_problem(node, "The vessel '" + vessel.name + "' doesn't have the userkey for user " + user.username + ".")

  finally:
    # We didn't do any locking if this readonly was True.
    if not readonly:
      
      # Release the lock
      lockserver.unlock_node(lockserver_handle, node.node_identifier)
      
      # Destroy the lockserver handle if we created it ourselves.
      if must_destroy_lockserver_handle:
        lockserver.destroy_lockserver_handle(lockserver_handle)




"""
<Program>
  ut_nodestatetransitions_test_onepercentmanyevents_to_canonical.py

<Purpose>
  Test the backend process transition_onepercentmanyevents_to_canonical.py.
  This is done in three steps. First we test the process of moving nodes
  from the onepercentmanyevents state to the movingto_canonical state,
  then we test the process of nodes moving from the movingto_canonical
  state to the canonical state. Then we test the process of nodes moving
  from the movingto_canonical state to the onepercentmanyevents state.

<Author>
  Monzur Muhammad
  monzum@cs.washington.edu

<Started>
  November 16, 2010
"""


#pragma out

# The seattlegeni testlib must be imported first.
from seattlegeni.tests import testlib

from seattlegeni.node_state_transitions import node_transition_lib

from seattlegeni.node_state_transitions import transition_onepercentmanyevents_to_canonical
from seattlegeni.common.api import maindb

from seattlegeni.node_state_transitions.tests import mockutil

from seattle.repyportability import *
add_dy_support(locals())

dy_import_module_symbols("rsa.r2py")





#vessel dictionary for this test
vessels_dict_basic = {}
vessels_dict_basic[mockutil.extra_vessel_name] = {"userkeys" : [node_transition_lib.transition_state_keys['onepercentmanyevents']],
                                                  "ownerkey" : mockutil.per_node_key,
                                                  "ownerinfo" : "",
                                                  "status" : "",
                                                  "advertise" : True}
vessels_dict_basic["vessel_non_seattlegeni"] = {"userkeys" : [node_transition_lib.transition_state_keys['onepercentmanyevents']],
                                                "ownerkey" : mockutil.donor_key,
                                                "ownerinfo" : "",
                                                "status" : "",
                                                "advertise" : True}
vessels_dict_basic["random_vessel"] = {"userkeys" : ["some random key"],
                                       "ownerkey" : mockutil.donor_key,
                                       "ownerinfo" : "",
                                       "status" : "",
                                       "advertise" : True}



# The vessels dict we will populate in setup.
vessels_dict = {}


def setup_general():
  """
  <Purpose>
    Prepare all the general stuff in order to run the tests.

  <Arguments>
    None

  <Exceptions>
    None

  <Side Efects>
    None

  <Return>
    None
  """

  testlib.setup_test_db()

  # Make sure all the variables are its default values
  mockutil.mockutil_cleanup_variables()

  # Create a user who has the donation key.
  user_object = maindb.create_user(mockutil.testusername, "password", "example@example.com", "affiliation",
                    "10 11", "2 2 2", mockutil.donor_key_str)

  # Create a database entry for the node
  node_object = maindb.create_node(mockutil.nodeid_key_str, mockutil.node_ip, mockutil.node_port, "10.0test",
                                  True, mockutil.per_node_key_str, mockutil.extra_vessel_name)


  # Create a donation for user
  maindb.create_donation(node_object, user_object, "Making a donation")

  testuser = maindb.get_user(mockutil.testusername)

  # Retrieve the info about the donation we just made.
  all_donations = maindb.get_donations_by_user(testuser, include_inactive_and_broken=True)
  node = all_donations[0].node

  global vessels_dict
  vessels_dict = vessels_dict_basic.copy()
  # Create 9 different vessels, assuming that the original vessel got split into 9.
  for i in range(9):
    vessels_dict["vessel"+str(i)]={}
    vessels_dict["vessel"+str(i)]["userkeys"] = []
    vessels_dict["vessel"+str(i)]["ownerkey"] = rsa_string_to_publickey(node.owner_pubkey)
    vessels_dict["vessel"+str(i)]["ownerinfo"] = ""
    vessels_dict["vessel"+str(i)]["status"] = ""
    vessels_dict["vessel"+str(i)]["advertise"] = True

    maindb.create_vessel(node, "vessel"+str(i))


  # Setup all the mock functions
  mockutil.mock_nodemanager_get_node_info(mockutil.nodeid_key, "10.0test", vessels_dict)
  mockutil.mock_lockserver_calls()
  mockutil.mock_backend_generate_key([mockutil.per_node_key_str])
  mockutil.mock_nodemanager_get_vesselresources()
  mockutil.mock_transitionlib_do_advertise_lookup([mockutil.node_address])
  mockutil.mock_backend_set_vessel_owner_key()




def setup_onepercentmanyevents_to_moving_to_canonical():
  """
  <Purpose>
    Setup everything thats needed to run the
    onepercentmanyevents_to_movingto_canonical test.

  <Arguments>
    None.

  <Side Effects>
    None.

  <Return>
    None.
  """
  mockutil.mock_backend_set_vessel_user_keylist([mockutil._mock_pubkey_to_string(
        node_transition_lib.transition_state_keys['movingto_canonical'])])





def setup_movingto_canonical_to_onepercent():
  """
  <Purpose>
    Setup everything thats needed to run the 
    movingto_canonical_to_onepercent test.

  <Arguments>
    None.

  <Side Effects>
    None.

  <Return>
    None.
  """

  testuser = maindb.get_user(mockutil.testusername)

  # Retrieve the info about the donation we just made.
  all_donations = maindb.get_donations_by_user(testuser, include_inactive_and_broken=True)
  node = all_donations[0].node

  # Delete all the vessel records from the database. Assume that the nodes were
  # joined back by this point.
  maindb.delete_all_vessels_of_node(node)

  # Do the setup for this test.
  vessels_dict[mockutil.extra_vessel_name]['userkeys'] = [node_transition_lib.transition_state_keys['movingto_canonical']]
  mockutil.mock_nodemanager_get_node_info(mockutil.nodeid_key, "10.0test", vessels_dict)
  mockutil.mock_backend_split_vessel()
  mockutil.mock_backend_set_vessel_user_keylist([mockutil._mock_pubkey_to_string(
                                                node_transition_lib.transition_state_keys['onepercentmanyevents'])])






def setup_movingto_canonical_to_canonical():
  """
  <Purpose>
    Setup everything thats needed to run the
    movingto_canonical_to_canonical test.

  <Arguments>
    None.

  <Side Effects>
    None.

  <Return>
    None.
  """

  # Do the setup for this test.
  vessels_dict[mockutil.extra_vessel_name]['userkeys'] = [node_transition_lib.transition_state_keys['movingto_canonical']]
  mockutil.mock_nodemanager_get_node_info(mockutil.nodeid_key, "10.0test", vessels_dict)
  mockutil.mock_backend_join_vessels()
  mockutil.mock_backend_set_vessel_user_keylist([mockutil._mock_pubkey_to_string(
                                                node_transition_lib.transition_state_keys['canonical'])])





def run_onepercentmanyevents_to_movingto_canonical():
  """
  <Purpose>
    Test the process of transitioning a node from the 
    onepercentmanyevents state to the movingto_canonical
    state.

  <Arguments>
    None

  <Side Effects>
    None

  <Exceptions>
    AssertionError raised if test fails.

  <Return>
    None
  """

  print "Starting onepercentmanyevents to movingto_canonical test....."
  transitionlist = []

  transitionlist.append(("onepercentmanyevents", "movingto_canonical",
                         node_transition_lib.noop,
                         node_transition_lib.noop, False))



  (success_count, failure_count) = node_transition_lib.do_one_processnode_run(transitionlist, "startstatename", 1)[0]

  assert(success_count == 1)
  assert(failure_count == 0)

  assert_database_info_non_active()

  assert(mockutil.set_vessel_owner_key_call_count == 0)
  assert(mockutil.set_vessel_user_keylist_call_count == 1)





def run_movingto_canonical_to_canonical():
  """
  <Purpose>
    Test the process of transitioning a node from the
    movingto_canonical state to the canonical
    state.

  <Arguments>
    None

  <Side Effects>
    None

  <Exceptions>
    AssertionError raised if test fails.

  <Return>
    None
  """

  print "Starting movingto_canonical to canonical test....."
  transitionlist = []

  transitionlist.append(("movingto_canonical", "canonical",
                         node_transition_lib.combine_vessels,
                         node_transition_lib.noop, False))



  (success_count, failure_count) = node_transition_lib.do_one_processnode_run(transitionlist, "startstatename", 1)[0]

  assert(success_count == 1)
  assert(failure_count == 0)

  assert_database_info_non_active()

  assert(mockutil.set_vessel_owner_key_call_count == 0)
  assert(mockutil.set_vessel_user_keylist_call_count == 1)

  # Retrieve the donated node and check its status with the database.
  testuser = maindb.get_user(mockutil.testusername)
  all_donations = maindb.get_donations_by_user(testuser, include_inactive_and_broken=True)
  node = all_donations[0].node

  vessel_list_per_node = maindb.get_vessels_on_node(node)

  #testing to see if the vessels were deleted after join_vessels was called
  assert(mockutil.join_vessels_call_count == 9)
  assert(len(vessel_list_per_node) == 0)
  assert(node.extra_vessel_name == "extra_vessel_join9")






def run_movingto_canonical_to_onepercentmanyevents():
  """
  <Purpose>
    Test the process of transitioning a node from the
    movingto_canonical state to the onepercentmanyevents
    state.

  <Arguments>
    None

  <Side Effects>
    None

  <Exceptions>
    AssertionError raised if test fails.

  <Return>
    None
  """

  print "Starting movingto_canonical to onepercentmanyevents test....."
  transitionlist = []

  onepercentmanyevents_resource_fd = file(transition_onepercentmanyevents_to_canonical.RESOURCES_TEMPLATE_FILE_PATH)
  onepercentmanyevents_resourcetemplate = onepercentmanyevents_resource_fd.read()
  onepercentmanyevents_resource_fd.close()

  transitionlist.append(("movingto_canonical", "onepercentmanyevents",
                         node_transition_lib.split_vessels,
                         node_transition_lib.noop, True, 
                         onepercentmanyevents_resourcetemplate))



  (success_count, failure_count) = node_transition_lib.do_one_processnode_run(transitionlist, "startstatename", 1)[0]

  assert(success_count == 1)
  assert(failure_count == 0)

  assert_database_info_active()

  assert(mockutil.set_vessel_owner_key_call_count == 0)

  # The count for setting user key list is 10, once for the extra vessel
  # and 9 times for splitting the vessel.
  assert(mockutil.set_vessel_user_keylist_call_count == 10)

  # Retrieve the donated node and check its status with the database.
  testuser = maindb.get_user(mockutil.testusername)
  all_donations = maindb.get_donations_by_user(testuser, include_inactive_and_broken=True)
  node = all_donations[0].node

  vessel_list_per_node = maindb.get_vessels_on_node(node)

  # Testing to see if the vessels were created after split_vessels was called
  assert(mockutil.split_vessel_call_count == 9)
  assert(len(vessel_list_per_node) == 9)

  for i in range(len(vessel_list_per_node)):
    # Note that the vessel names go from 1-9 rather then 0-8
    assert(vessel_list_per_node[i].node == node)
    assert(vessel_list_per_node[i].name == "new_vessel"+str(1+i))

  assert(node.extra_vessel_name == "extra_vessel_split9")





def assert_database_info_non_active():

  active_nodes_list = maindb.get_active_nodes()
  assert(len(active_nodes_list) == 0)

  testuser = maindb.get_user(mockutil.testusername)

  active_donations = maindb.get_donations_by_user(testuser)
  assert(len(active_donations) == 0)

  all_donations = maindb.get_donations_by_user(testuser, include_inactive_and_broken=True)
  assert(len(all_donations) == 1)

  node = all_donations[0].node

  assert(node.node_identifier == mockutil.nodeid_key_str)
  assert(node.last_known_ip == mockutil.node_ip)
  assert(node.last_known_port == mockutil.node_port)
  assert(node.extra_vessel_name == mockutil.extra_vessel_name)
  assert(node.owner_pubkey == mockutil.per_node_key_str)





def assert_database_info_active():

  active_nodes_list = maindb.get_active_nodes()

  assert(len(active_nodes_list) == 1)
  assert(active_nodes_list[0].node_identifier == mockutil.nodeid_key_str)
  assert(active_nodes_list[0].last_known_ip == mockutil.node_ip)
  assert(active_nodes_list[0].last_known_port == mockutil.node_port)
  assert(active_nodes_list[0].extra_vessel_name == mockutil.extra_vessel_name)
  assert(active_nodes_list[0].owner_pubkey == mockutil.per_node_key_str)

  testuser = maindb.get_user(mockutil.testusername)
  donations_list = maindb.get_donations_by_user(testuser)

  assert(len(donations_list) == 1)
  assert(donations_list[0].node == active_nodes_list[0])





def teardown_test():

  # Cleanup the test database.
  testlib.teardown_test_db()






if __name__ == "__main__":
  setup_general()
  setup_onepercentmanyevents_to_moving_to_canonical()
  try:
    run_onepercentmanyevents_to_movingto_canonical()
  finally:
    teardown_test()

  setup_general()
  setup_movingto_canonical_to_canonical()
  try:
    run_movingto_canonical_to_canonical()
  finally:
    teardown_test()

  setup_general()
  setup_movingto_canonical_to_onepercent()
  try:
    run_movingto_canonical_to_onepercentmanyevents()
  finally:
    teardown_test()
  print "All Tests Passed!"



"""
<Program>
  keydb.py

<Started>
  29 June 2009

<Author>
  Justin Samuel

<Purpose>
   This is the API that should be used to interact with the Key Database.
   Functions in this module are the only way that other code should interact
   with the Key Database.
   
   The init_keydb() method must be called before calling the other methods.
   
   For info on setting up and security access to the keydb, see the file
   seattlegeni/keydb/README.txt.
"""

import MySQLdb
import traceback

from seattlegeni.common.exceptions import *

from seattlegeni.common.util.assertions import *

from seattlegeni.keydb import config





def init_keydb():
  """
  <Purpose>
    Initialize the keydb api.
  <Arguments>
    None
  <Exceptions>
    None
  <Side Effects>
    Does nothing for the moment, actually. Maybe someday it will setup a
    connection pool or do other important initialization on the database.
    We can't just make a single connection and create a new cursor for
    each request because that's not how cursors work, at least with MySQLdb.
  <Returns>
    None
  """
  pass  





def _get_connection():
  try:
    return MySQLdb.connect(user=config.dbuser, passwd=config.dbpass, db=config.dbname, host=config.dbhost)
  except MySQLdb.Error:
    raise InternalError("Failed initializing key database: " + traceback.format_exc())





def _release_connection(connection):
  connection.close()





def get_private_key(pubkey):
  """
  <Purpose>
    Get a private key from the key database.
  <Arguments>
    pubkey
      The public key whose corresponding private key should be returned.
  <Exceptions>
    DoesNotExistError
      If there is no private key stored that corresponds to pubkey.
  <Side Effects>
    None
  <Returns>
    The private key that corresponds to pubkey.
  """
  
  assert_str(pubkey)
  
  connection = _get_connection()
  cursor = connection.cursor()
  
  try:
    try:
      # Note: `keys` is a reserved mysql keyword so must be quoted.
      cursor.execute("SELECT privkey FROM `keys` WHERE pubkey = %s", (pubkey))
    except MySQLdb.Error:
      raise InternalError("Failed getting private key: " + traceback.format_exc())
   
    if cursor.rowcount != 1:
      raise DoesNotExistError("No private key corresponding to the public key: " + pubkey)
    
    return cursor.fetchone()[0]
  
  finally:
    cursor.close()
    _release_connection(connection)





def set_private_key(pubkey, privkey, keydescription):
  """
  <Purpose>
    Store a private key in the key database.
  <Arguments>
    pubkey
      The public key to be stored.
    privkey
      The private key to be stored.
    keydescription
      A description of what this key is used for. E.g. "donor:[usernamehere]"
      to indicate the donor key for a specific user. This is non-critical and
      only because it may come in handy later.
  <Exceptions>
    None
  <Side Effects>
    A new record is added to the keys table in the key database with the
    information specified.
  <Returns>
    None
  """
  
  assert_str(pubkey)
  assert_str(privkey)
  assert_str(keydescription)
  
  connection = _get_connection()
  cursor = connection.cursor()
  
  try:
    try:
      # We insert include the pubkeyhash field to ensure that keys are unique.
      # We can't use the pubkey field because it is too long to enforce
      # uniqueness at the database level. We don't want to try to query it
      # ourselves because then we need to do two queries and have a shared access
      # lock around them, and that's a lot messier. This should be an uncommon
      # error case and likely means there's something very wrong in some other
      # part of the system, sothis is the simplest way to raise an exception if
      # this case does occur.
      # Note: `keys` is a reserved mysql keyword so must be quoted.
      cursor.execute("INSERT INTO `keys` (pubkeyhash, pubkey, privkey, description) VALUES (MD5(%s), %s, %s, %s)", 
                     (pubkey, pubkey, privkey, keydescription))
    except MySQLdb.Error:
      raise InternalError("Failed setting private key: " + traceback.format_exc())
  
  finally:
    cursor.close()
    _release_connection(connection)




"""
Provides information about the data in the database. This is for generating
reports, not for any core functionality of seattlegeni.

Some of the functions in the module are very database intensive. They could
be done more efficiently, but where possible this module tries to use the
maindb api so that summarized information matches how seattlegeni actually
sees things.
"""

from seattlegeni.common.api import maindb

from seattlegeni.website.control.models import GeniUser

from seattlegeni.common.util import log






def get_vessel_acquisition_counts_by_user():

  # Set the log level high enough so that we don't produce a bunch of logging
  # output due to the logging decorators.
  initial_log_level = log.loglevel
  log.set_log_level(log.LOG_LEVEL_INFO)
  
  vessel_acquisition_dict = {}
  
  for user in GeniUser.objects.all():
    acquired_vessels = maindb.get_acquired_vessels(user)
    if len(acquired_vessels) > 0:
      vessel_acquisition_dict[user.username] = len(acquired_vessels)
      
  # Restore the original log level.
  log.set_log_level(initial_log_level)
      
  return vessel_acquisition_dict





def get_donation_counts_by_user():
  
  # Set the log level high enough so that we don't produce a bunch of logging
  # output due to the logging decorators.
  initial_log_level = log.loglevel
  log.set_log_level(log.LOG_LEVEL_INFO)
  
  donation_dict = {}
  
  for user in GeniUser.objects.all():
    active_donation_count = len(maindb.get_donations_by_user(user))
    inactive_donation_count = len(maindb.get_donations_by_user(user, include_inactive_and_broken=True)) - active_donation_count
    if active_donation_count > 0 or inactive_donation_count > 0:
      donation_dict[user.username] = (active_donation_count, inactive_donation_count)
      
  # Restore the original log level.
  log.set_log_level(initial_log_level)
      
  return donation_dict





def get_available_vessel_counts_by_port():

  # Set the log level high enough so that we don't produce a bunch of logging
  # output due to the logging decorators.
  initial_log_level = log.loglevel
  log.set_log_level(log.LOG_LEVEL_INFO)

  available_vessels_dict = {}
  
  for port in maindb.ALLOWED_USER_PORTS:
    available_vessels_dict[port] = {}
    available_vessels_dict[port]["all"] = maindb._get_queryset_of_all_available_vessels_for_a_port_include_nat_nodes(port).count()
    available_vessels_dict[port]["no_nat"] = maindb._get_queryset_of_all_available_vessels_for_a_port_exclude_nat_nodes(port).count()
    available_vessels_dict[port]["only_nat"] = maindb._get_queryset_of_all_available_vessels_for_a_port_only_nat_nodes(port).count()
  
  # Restore the original log level.
  log.set_log_level(initial_log_level)
  
  return available_vessels_dict





def get_available_lan_vessel_counts():
  """
  Returns a dictionary where each key is a port and each value is a
  reverse-sorted list of integers representing the the sizes of subnets with
  available vessels on that port. These will be the maximum numbers of LAN
  vessels a user on that port could request at once.
  """
  
  lan_sizes_by_port = {}

  subnetlist = maindb._get_subnet_list()

  for port in maindb.ALLOWED_USER_PORTS:
    subnet_vessel_list_sizes = []
    nonnatvesselsqueryset = maindb._get_queryset_of_all_available_vessels_for_a_port_exclude_nat_nodes(port)
  
    for subnet in subnetlist:
      lanvesselsqueryset = nonnatvesselsqueryset.filter(node__last_known_ip__startswith=subnet + '.')
      subnet_vessel_list_sizes.append(lanvesselsqueryset.count())
  
    subnet_vessel_list_sizes.sort(reverse=True)
    lan_sizes_by_port[port] = subnet_vessel_list_sizes

  return lan_sizes_by_port



"""
<Program>
  exceptions.py

<Started>
  6 July 2009

<Author>
  Justin Samuel
  
<Purpose>
  All exceptions used outside of a single module in seattlegeni are defined
  in this file. All seattlegeni modules should import all exceptions in this
  file. That is, the following code should be at the top of all seattlegeni
  modules:
  
  from seattlegeni.common.exceptions import *
  
  This ensures that we don't end up ever having an exception named in a
  try/catch block that isn't actually defined in that modules namespace.
  
  Note: We define __all__ at the bottom of this script to make sure that we
  only export the exceptions we've defined here, not anything else that may be
  imported or used in this module. Only names that end in "Error" are exported.
  
  In general, no code in seattlegeni should knowingly let built-in
  python exceptions escape from the place where the exception occurs.
  Instead, it should be caught and, if it can't be dealt with, re-raised as
  one of the exceptions in this module (with the details of the original
  exception in the message). Generally this will be either raising the
  exception as a ProgrammerError or an InternalError. That is, if it's
  something we can't recover from, it's usually either bad code or something
  broken/a service down.
"""

# TODO: allow construction of the exceptions and passing them another exception as
# an argument, where the details of that other exception will also be printed
# when the new exception's details are.

class SeattleGeniError(Exception):
  """
  <Purpose>
    All other custom exceptions of seattlegeni inherit from this.
  """



class InvalidRequestError(SeattleGeniError):
  """
  Indicates that a requested action was invalid. This is an intentionally
  generic error. This error should be raised with a message that clearly
  explains what was invalid.
  """



class ProgrammerError(SeattleGeniError):
  """
  Indicates that a programmer is using something incorrectly. Rather than
  extend this class for many different cases, this error should be raised
  with a message that clearly explains what the programmer did wrong
  (for example, they passed an argument of the wrong type into a function).
  
  Do not catch this exception! You may prevent proper notification about a
  broken part of seattlegeni. This exception will never be documented that
  it can be raised. You should assume it's always possible that it be
  raised. If you really know what you're doing, you can break this rule and
  catch a ProgrammerError.
  """
  

  
class InternalError(SeattleGeniError):
  """
  Indicates that some part of the seattlegeni system failed. E.g., a
  communication problem with the lockserver, backend, or database. Can also
  indicate the database is in a bad state. The text of the raised exception
  should clearly describe the problems and all related details.
  
  Do not catch this exception! You may prevent proper notification about a
  broken part of seattlegeni. This exception will never be documented that
  it can be raised. You should assume it's always possible that it be
  raised. If you really know what you're doing, you can break this rule and
  catch an InternalError.
  """



class NodemanagerCommunicationError(SeattleGeniError):
  """
  Indicates a failure in communication with a nodemanager.
  """



class DoesNotExistError(SeattleGeniError):
  """
  Indicates that some requested data/record does not exist.
  """



class ValidationError(SeattleGeniError):
  """
  Indicates that some data checked for validity is invalid. This is not the
  same as a django.forms.ValidationError.
  """



class UsernameAlreadyExistsError(SeattleGeniError):
  """
  Indicates that registration of a username was attempted but that there
  is already a user with this username.
  """
  
  
  
class UnableToAcquireResourcesError(SeattleGeniError):
  """
  Indicates that SeattleGeni was unable to satisfy a request for resource
  acquisition.
  """

  
  
class InsufficientUserResourcesError(SeattleGeniError):
  """
  Indicates that a user requested more resources than they are allowed to have.
  """



class TimeUpdateError(SeattleGeniError):
  """
  Indicates that a call to the time_updatetime() repy function failed. This is
  to avoid this situation being treated as an InternalError in case it's
  considered to be non-fatal at some point in the future.
  """



# Many modules are using the line 'from seattlegeni.common.exceptions import *'
# to import the exceptions. We define __all__ so that we only export names from
# this module that end in the word "Error". 
__all__ = []
for name in dir():
  if name.endswith("Error"):
    __all__.append(name)




"""
<Program Name>
  deploy_seattlegeni.py

<Started>
  July 29, 2009

<Author>
  Justin Samuel

<Purpose>
  Deploys all components of seattlegeni. Some things (e.g. database setup,
  config file modification to enter db user/pass info, etc.) still need
  to be done after this is run. See the instructions in:
  seattlegeni/README.txt

<Description>
  The script backs up the target deploy directory if it exists and the user
  wants to proceed. It then deploys all required files to the deploy
  directory the user specified.

<Usage>
  python deploy_seattlegeni.py <path/to/svn/trunk> <dir/to/deploy/to>
  
"""

import glob
import os
import shutil
import subprocess
import sys
import time





# This function was taken from preparetest. It seemed annoying to depend on
# preparetest just for this. If we use preparetest for anything else later on,
# (e.g. preparing tests?), then we should remove this.
def _copy_to_target(file_expr, target):
  """
  file_expr may contain wildcards
  target must specify an existing directory with no wildcards
  """
  files_to_copy = glob.glob(file_expr)
  for file_path in files_to_copy:
    if os.path.isfile(file_path):
      shutil.copyfile(file_path, target + "/" + os.path.basename(file_path))





def _print_post_deploy_instructions():
  print
  print "Deployed successfully. The file trunk/seattlegeni/README.txt contains the"
  print "rest of the info you'll need to get things running."





def _deploy_seattle_files_to_directory(trunkdir, targetdir):
  """
  Deploys a copy of all seattle files needed to run repy code to a specified
  directory.
  """
  
  print "Deploying seattle and repy library code to " + targetdir
  
  # Copy the repy files needed by various parts of seattlegeni,
  # including ones we don't use but may be required to import repyhelper 
  # (OBSOLETE?).
  _copy_to_target(os.path.join(trunkdir, "repy", "*"), targetdir)
  _copy_to_target(os.path.join(trunkdir, "nodemanager", "*"), targetdir)
  _copy_to_target(os.path.join(trunkdir, "portability", "*"), targetdir)
  _copy_to_target(os.path.join(trunkdir, "seattlelib", "*"), targetdir)

  _process_mix_files_in_directory(trunkdir, targetdir)





#iterate through the .mix files in current folder and run them through the preprocessor
#script_path must specify the name of the preprocessor script
#the working directory must be set to the directory containing the preprocessor script prior to executing this function.
def _process_mix_files_in_directory(trunkdir, directory_with_mix_files):
 
  originaldir = os.getcwd()
  os.chdir(directory_with_mix_files)

  mix_files = glob.glob("*.mix")
 
  # Generate a .py file for each .mix file.
  for file_path in mix_files:
    processed_file_path = (os.path.basename(file_path)).replace(".mix", ".py")
    retval = subprocess.call(["python", "repypp.py", file_path, processed_file_path])
    if retval != 0:
      exit_with_message(1, "Failed converting " + file_path + " to " + processed_file_path)

  os.chdir(originaldir)





def main():
    
  if not len(sys.argv) == 3:
    exit_with_message(2, "Usage: python deploy_seattlegeni.py <path/to/svn/trunk> <dir/to/deploy/to>")
  
  trunkdir = sys.argv[1]
  deployroot = sys.argv[2]
  
  if not os.path.isdir(trunkdir):
    exit_with_message(1, "ERROR: the provided path to the svn trunk does not exist.")

  if not os.path.exists(os.path.join(trunkdir, "seattlegeni")):
    exit_with_message(1, "ERROR: the given svn trunk directory doesn't contains a seattlegeni directory.")

  # Warn the user if the provided deploy directory exists and if it should be replaced.
  # We actually rename rather than remove the old one, just to be paranoid.
  replace_deployment_dir = False
  if os.path.isdir(deployroot):
    print "WARNING: existing directory found at directory to deploy: " + deployroot
    ans = raw_input("Backup and replace this directory (copying config files to the new deployment)? [y/n]")
    replace_deployment_dir = str.lower(ans) == 'y'
    if not replace_deployment_dir:
      exit_with_message(2, "You chose not to replace the directory.")
    else:
      print
      renameddir = deployroot.rstrip('/').rstrip('\\') + '.bak.' + str(time.time())
      print "Renaming existing directory " + deployroot + " to " + renameddir
      shutil.move(deployroot, renameddir)

  # Create the directory we will deploy to.
  print "Creating directory " + deployroot
  os.mkdir(deployroot)

  # Copy over the seattlegeni files from svn to the deploy directory.
  #seattlegeni_svn = os.path.join(trunkdir, "seattlegeni", "website")
  seattlegeni_svn_dir = os.path.join(trunkdir, "seattlegeni")
  seattlegeni_deploy_dir = os.path.join(deployroot, "seattlegeni")
  print "Copying " + seattlegeni_svn_dir + " to " + seattlegeni_deploy_dir
  shutil.copytree(seattlegeni_svn_dir, seattlegeni_deploy_dir, symlinks=True)

  # Deploy the seattle/repy library files in a directory called "seattle". This
  # will serve. This will be the sole location of repy files in seattlegeni and
  # will also serve as a python package called "seattle" for use within
  # seattlegeni code.
  seattle_package_dir = os.path.join(deployroot, "seattle")
  os.mkdir(seattle_package_dir)
  # Open/close file to emulate a "touch __init__.py"
  open(os.path.join(seattle_package_dir, "__init__.py"), "a").close()
  _deploy_seattle_files_to_directory(trunkdir, seattle_package_dir)
  
  # If we replaced an existing directory, then copy the config files from the
  # old deployment to the new one.
  if replace_deployment_dir:
    # The website settings files.
    old_settings_path = os.path.join(renameddir, "seattlegeni", "website", "settings.py")
    new_settings_path = os.path.join(seattlegeni_deploy_dir, "website", "settings.py")
    print "Copying " + old_settings_path + " to " + new_settings_path
    # Use copy2 to preserve permissions, e.g. in case these weren't world-readable.
    shutil.copy2(old_settings_path, new_settings_path)
    
    # The keydb config file.
    old_keydb_config_path = os.path.join(renameddir, "seattlegeni", "keydb", "config.py")
    new_keydb_config_path = os.path.join(seattlegeni_deploy_dir, "keydb", "config.py")
    print "Copying " + old_keydb_config_path + " to " + new_keydb_config_path
    # Use copy2 to preserve permissions, e.g. in case these weren't world-readable.
    shutil.copy2(old_keydb_config_path, new_keydb_config_path)

    # The backend config file.
    old_keydb_config_path = os.path.join(renameddir, "seattlegeni", "backend", "config.py")
    new_keydb_config_path = os.path.join(seattlegeni_deploy_dir, "backend", "config.py")
    print "Copying " + old_keydb_config_path + " to " + new_keydb_config_path
    # Use copy2 to preserve permissions, e.g. in case these weren't world-readable.
    shutil.copy2(old_keydb_config_path, new_keydb_config_path)

  state_key_path = os.path.join(seattlegeni_deploy_dir, "node_state_transitions", "statekeys")
  beta_state_key_path = os.path.join(seattlegeni_deploy_dir, "node_state_transitions", "statekeys_beta")
  
  # Remove the production key from the deployment, as we don't want anyone to get confused by them.
  # If there is any error, just ignore it.
  try:
    shutil.rmtree(state_key_path, ignore_errors=True)
    shutil.rmtree(beta_state_key_path, ignore_errors=True)
  except:
    pass

  _print_post_deploy_instructions()





def exit_with_message(retval, message):
  print message
  print "Exiting."
  sys.exit(retval)





if __name__ == '__main__':
  main()



"""
<Program Name>
  interface.py

<Started>
  June 17, 2009

<Author>
  Justin Samuel

<Purpose>
  This module presents the only methods that a frontend action will call. These
  methods do the work of ensuring that an action requested through a frontend
  are performed. The resulting data that the frontend needs is returned from
  these methods.
  
  This module should be the only point of entry from a frontend to the rest of
  the code base.

  Functions in this module will make calls to the following APIs:
    * backend
    * keygen
    * lockserver
    * maindb
  
<Notes>
  * All references to user here are to our GeniUser model, not to the django user.
  
  * The functions that modify the seattlegeni database or perform actions on
    nodes all do an extra check to ensure the user is valid after obtaining a
    user lock. This is to ensure that user has not been deleted and to see other
    changes to the user that were made between the time that the request was
    made and when the lock was obtained.
    
  * When using this module from the frontend views, you do not need to catch
    InternalError, ProgrammerError, or otherwise uncaught exceptions. Those
    are allowed to trickle all the way up and get handled based on how we've
    configured django.
"""

import traceback
import datetime

import django.contrib.auth

from seattlegeni.common.exceptions import *

from seattlegeni.common.api import backend
from seattlegeni.common.api import keygen
from seattlegeni.common.api import lockserver
from seattlegeni.common.api import maindb

from seattlegeni.common.util import validations

from seattlegeni.common.util.assertions import *

from seattlegeni.common.util.action_log_decorators import log_action

from seattlegeni.common.util.decorators import log_function_call
from seattlegeni.common.util.decorators import log_function_call_and_only_first_argument
from seattlegeni.common.util.decorators import log_function_call_without_arguments
from seattlegeni.common.util.decorators import log_function_call_without_return

from seattlegeni.website.control import vessels





@log_function_call_and_only_first_argument
def register_user(username, password, email, affiliation, pubkey=None):
  """
  <Purpose>
    Creates a user record with the specified information and sets any additional
    information necessary for the user record to be complete.
  <Arguments>
    username
    password
    email
    affiliation
    pubkey
      Optional. A string. If not provided, a key pair will be generated for this user.
  <Exceptions>
    UsernameAlreadyExistsError
      If there is already a user with the specified username.
    ValidationError
      If any of the arguments contains invalid values or if the username is the
      same as the password.
  <Side Effects>
    The user record in the django db is created as well as a user record in the
    corresponding user profile table that stores our custom information. A port
    will be assigned to the user and the user's donation keys will be set.
  <Returns>
    GeniUser instance (our GeniUser model, not the django User) corresponding to the
    newly registered user.
  """
  # If the frontend code that called this function wants to know which field
  # is invalid, it must call the validation functions itself before making the
  # call to register_user().
  # These will raise a ValidationError if any of the fields are invalid.
  # These ensure that the data is of the correct type (e.g. a string) as well as
  # that we like the content of the variable.
  validations.validate_username(username)
  validations.validate_password(password)
  validations.validate_username_and_password_different(username, password)
  validations.validate_email(email)
  validations.validate_affiliation(affiliation)
  if pubkey is not None:
    validations.validate_pubkey_string(pubkey)
  
  # Lock the user.
  lockserver_handle = lockserver.create_lockserver_handle()
  lockserver.lock_user(lockserver_handle, username)
  try:
    # Ensure there is not already a user with this username.
    try:
      # Raises a DoesNotExistError if the user doesn't exist.
      maindb.get_user(username)
      raise UsernameAlreadyExistsError
    except DoesNotExistError:
      # This is what we wanted: the username isn't already taken.
      pass
    
    # Get a key pair from the keygen api if the user didn't supply their own pubkey.
    if pubkey is None:
      (pubkey, privkey) = keygen.generate_keypair()
    else:
      privkey = None
    
    # Generate a donor key for this user. This is done through the backend
    # as the private key must be stored in the keydb, which the website cannot
    # directly access.
    keydescription = "donor:" + username
    donor_pubkey = backend.generate_key(keydescription)
    
    # Create the user record.
    geniuser = maindb.create_user(username, password, email, affiliation, pubkey, privkey, donor_pubkey)
    
  finally:
    # Unlock the user.
    lockserver.unlock_user(lockserver_handle, username)
    lockserver.destroy_lockserver_handle(lockserver_handle)
    
  return geniuser
  



#def change_user_pubkey(geniuser, user_pubkey):
#  """
#  <Purpose>
#    Assigns a new key pair to this user.
#  <Arguments>
#    geniuser
#    user_pubkey
#  <Exceptions>
#    ProgrammerError
#      If the user is not a valid GeniUser object or if user_pubkey is not a
#      valid pubkey.
#  <Side Effects>
#    The user's public key is replaced with the provided one. If they had a
#    private key still stored in our database, that private key is deleted.
#  <Returns>
#    None
#  """
#  raise NotImplementedError




# JTC: Added for installers.
@log_function_call
def get_user_for_installers(username):
  """
  <Purpose>
    Gets the user record corresponding to the given username.
    IMPORTANT: Used ONLY FOR getting the user object for downloading/building installers.
               Do NOT use for any other purpose, as this function does not validate passwords.
  <Arguments>
    username
      The username (must be a string).
  <Exceptions>
    DoesNotExistError
      If there is no user with the specified username and password.
  <Side Effects>
    None
  <Returns>
    The GeniUser instance if the username is valid.
  """
  assert_str(username)
  
  return maindb.get_user(username)





@log_function_call
def get_user_without_password(username):
  """
  <Purpose>
    Gets the user record corresponding to the given username.
  <Arguments>
    username
      The username (must be a string).
  <Exceptions>
    DoesNotExistError
      If there is no user with the specified username.
  <Side Effects>
    None
  <Returns>
    The GeniUser instance if the username is valid.
  """
  assert_str(username)
  
  return maindb.get_user(username)





@log_function_call_and_only_first_argument
def get_user_with_password(username, password):
  """
  <Purpose>
    Gets the user record corresponding to the username and password.
  <Arguments>
    username
      The username (must be a string).
    password
      The password (must be a string).
  <Exceptions>
    DoesNotExistError
      If there is no user with the specified username and password.
  <Side Effects>
    None
  <Returns>
    The GeniUser instance if the username/password are valid.
  """
  assert_str(username)
  assert_str(password)
  
  return maindb.get_user_with_password(username, password)





@log_function_call_and_only_first_argument
def get_user_with_api_key(username, api_key):
  """
  <Purpose>
    Gets the user record corresponding to the username and api_key.
  <Arguments>
    username
      The username (must be a string).
    api_key
      The api_key (must be a string).
  <Exceptions>
    DoesNotExistError
      If there is no user with the specified username and api_key.
  <Side Effects>
    None
  <Returns>
    The GeniUser instance if the username/api_key are valid.
  """
  assert_str(username)
  assert_str(api_key)
  
  return maindb.get_user_with_api_key(username, api_key)  

  



# Not logging arguments. The call to get_user_with_password()
# made within this function will log those details.
@log_function_call_without_arguments
def login_user(request, username, password):
  """
  <Purpose>
    Log a user in to a session if the username and password are valid. This
    allows future requests that are part ofsame session to obtain the user
    object through calls to get_logged_in_user(). This function should not
    be used through the xmlrpc frontend as there is no concept of the session
    there.
  <Arguments>
    request
      The HttpRequest object of the user's request through the frontend.
    username
      The username of the user to be logged in.
    password
      The password of the user to be logged in.
  <Exceptions>
    DoesNotExistError
      If there is no user with the provided username and password.
  <Side Effects>
    Associates the user with a session corresponding to the request.
  <Returns>
    None
  """
  assert_str(username)
  assert_str(password)
  
  # Raises a DoesNotExistError if there is no such user. We could actually
  # skip this and only rely on authenticate, but I'd like to make sure that
  # the get_user_with_password function gets called before letting the user
  # access the site. The the note further down about checking the is_active
  # field for one example of why.
  get_user_with_password(username, password)
  
  # The auth.authenticate() method must be called before the auth.login()
  # method, as this method sets some values in the user object that are
  # required by auth.login().
  djangouser = django.contrib.auth.authenticate(username=username, password=password)
  
  # Note that we aren't checking the is_active field which is part of the base
  # django user class. If we want to be able to disable user accounts, we need
  # to be sure to do it in a way that will work for all frontends, including
  # xmlrpc. So, for example, the get_user_with_* methods could be changed.
  # If we looked for it only here, then the user could still perform actions
  # via xmlrpc.
  
  # Logs the user in via django's auth platform. This associates the user
  # with the current session. 
  django.contrib.auth.login(request, djangouser)
  




@log_function_call
def get_logged_in_user(request):
  """
  <Purpose>
    Determine the user logged in to the current session.
    This function should not be used through the xmlrpc frontend as there is
    no concept of the session there.
  <Arguments>
    request
      The HttpRequest object of the user's request through the frontend.
  <Exceptions>
    DoesNotExistError
      If there is no user logged in to the current session.
  <Side Effects>
    None
  <Returns>
    The GeniUser object of the logged in user.
  """
  # See http://docs.djangoproject.com/en/dev/topics/auth/#authentication-in-web-requests
  # for an explanation of request.user.is_authenticated().
  if request.user.is_authenticated():
    return maindb.get_user(request.user.username)
  else:
    raise DoesNotExistError





@log_function_call
def logout_user(request):
  """
  <Purpose>
    Logs out the user logged in to the current session, if any.
  <Arguments>
    request
      The HttpRequest object of the user's request through the frontend.
  <Exceptions>
    None
  <Side Effects>
    Any user logged in to the current session is no longer logged in.
  <Returns>
    None
  """
  request.session.flush()





@log_function_call
def change_user_keys(geniuser, pubkey=None):
  """
  <Purpose>
    Sets a new public/private key for the user and initiates the change
    of user keys on all vessels the user has access to. If pubkey is
    provided, that is used as the user's new pubkey. If pubkey is not
    provided, a new public/private keypair is generated for the user.
  <Arguments>
    geniuser
      A GeniUser object of the user whose keys are to be updated.
  <Exceptions>
    ValidationError
      If the pubkey is provided and is invalid.
  <Side Effects>
    The public and private keys of the user are replaced in the database with
    new keys (if pubkey was provided, the private key in the database will
    be empty, otherwise it will be the generated private key). All vessels the
    user has access to are marked as needing to have their user keys sync'd.
  <Returns>
    None
  """
  assert_geniuser(geniuser)
  
  if pubkey is not None:
    validations.validate_pubkey_string(pubkey)
  
  # Lock the user.
  lockserver_handle = lockserver.create_lockserver_handle()
  lockserver.lock_user(lockserver_handle, geniuser.username)
  try:
    # Make sure the user still exists now that we hold the lock. Also makes
    # sure that we see any changes made to the user before we obtained the lock.
    # We don't use the user object we retrieve because we want the
    # object passed in to the function to reflect changes we make to the object.
    try:
      maindb.get_user(geniuser.username)
    except DoesNotExistError:
      raise InternalError(traceback.format_exc())
    
    # Get a key pair from the keygen api if the user didn't supply their own pubkey.
    if pubkey is None:
      (pubkey, privkey) = keygen.generate_keypair()
    else:
      privkey = None    
    
    maindb.set_user_keys(geniuser, pubkey, privkey)
    
    # Now we need to find all of the vessels the user has access to and set
    # them to have their user keys updated by the backend.
    vessel_list = maindb.get_vessels_accessible_by_user(geniuser)
    if vessel_list:
      vessels.flag_vessels_for_user_keys_sync(lockserver_handle, vessel_list)
    
  finally:
    # Unlock the user.
    lockserver.unlock_user(lockserver_handle, geniuser.username)
    lockserver.destroy_lockserver_handle(lockserver_handle)




	
def get_useable_ports():
  """
  <Purpose>
     Gets the allowed user port range
  <Arguments>
	None
  <Exceptions>
   None
  <Side Effects>
    None
  <Returns>
    The allowed user port range which is globally defined in maindb.
  """
  return maindb.get_allowed_user_ports()





@log_function_call
def change_user_email(geniuser, new_email):
  """
  <Purpose>
     Sets a new email for the user 
  <Arguments>
    geniuser
      A GeniUser object of the user whose email is to be updated.
    new_email
      the new email value
  <Exceptions>
    ValidationError
      If the email is provided and is invalid.
  <Side Effects>
    The geniuser email gets changed to the new value(in the db).
  <Returns>
    None
  """
  assert_geniuser(geniuser)
  #validate its a real email.  The frontend should already
  #checks for this but we validate again just in case.
  validations.validate_email(new_email)
 
  # Lock the user.
  lockserver_handle = lockserver.create_lockserver_handle()
  lockserver.lock_user(lockserver_handle, geniuser.username)
  try:
    # Make sure the user still exists now that we hold the lock. Also makes
    # sure that we see any changes made to the user before we obtained the lock.
    # We don't use the user object we retrieve because we want the
    # object passed in to the function to reflect changes we make to the object.
    try:
      maindb.get_user(geniuser.username)
    except DoesNotExistError:
      raise InternalError(traceback.format_exc())

    maindb.set_user_email(geniuser, new_email)
  finally:
    # Unlock the user.
    lockserver.unlock_user(lockserver_handle, geniuser.username)
    lockserver.destroy_lockserver_handle(lockserver_handle)





@log_function_call
def change_user_affiliation(geniuser, new_affiliation):
  """
  <Purpose>
     Sets a new affiliation for the user 
  <Arguments>
    geniuser
      A GeniUser object of the user whose affiliation is to be updated.
    new_affiliation
      the new affiliation value
  <Exceptions>
    ValidationError
      If the affiliation is provided and is invalid.
  <Side Effects>
    The geniuser affiliation gets changed to the new value(in the db).
  <Returns>
    None
  """
  assert_geniuser(geniuser)
  #Determines if the new affiliation is valid.  The frontend should already
  #checks for this but we validate again here just in case.
  validations.validate_affiliation(new_affiliation)
 
  
  # Lock the user.
  lockserver_handle = lockserver.create_lockserver_handle()
  lockserver.lock_user(lockserver_handle, geniuser.username)
  try:
    # Make sure the user still exists now that we hold the lock. Also makes
    # sure that we see any changes made to the user before we obtained the lock.
    # We don't use the user object we retrieve because we want the
    # object passed in to the function to reflect changes we make to the object.
    try:
      maindb.get_user(geniuser.username)
    except DoesNotExistError:
      raise InternalError(traceback.format_exc())

    maindb.set_user_affiliation(geniuser, new_affiliation)
  finally:
    # Unlock the user.
    lockserver.unlock_user(lockserver_handle, geniuser.username)
    lockserver.destroy_lockserver_handle(lockserver_handle)





@log_function_call
def change_user_port(geniuser, new_port):
  """
  <Purpose>
     Sets a new port for the user 
  <Arguments>
    geniuser
      A GeniUser object of the user whose port is to be changed.
    new_port
      the new port value
  <Exceptions>
    ValidationError
      If the port is provided and it is not in the allowed range.
  <Side Effects>
    the geniuser port gets changed to the new value(in the db).
  <Returns>
    None
  """
  assert_geniuser(geniuser)

  # Lock the user.
  lockserver_handle = lockserver.create_lockserver_handle()
  lockserver.lock_user(lockserver_handle, geniuser.username)
  try:
    # Make sure the user still exists now that we hold the lock. Also makes
    # sure that we see any changes made to the user before we obtained the lock.
    # We don't use the user object we retrieve because we want the
    # object passed in to the function to reflect changes we make to the object.
    try:
      maindb.get_user(geniuser.username)
    except DoesNotExistError:
      raise InternalError(traceback.format_exc())

    maindb.set_user_port(geniuser, new_port)
  finally:
    # Unlock the user.
    lockserver.unlock_user(lockserver_handle, geniuser.username)
    lockserver.destroy_lockserver_handle(lockserver_handle)




	
@log_function_call_and_only_first_argument
def change_user_password(geniuser, new_password):
  """
  <Purpose>
     Sets a new password for the geniuser. 
  <Arguments>
    geniuser
      A GeniUser object of the user whose password is to be changed.
    new_password
      the user specificed new password value.
  <Exceptions>
    ValidationError
      If the password is provided and is invalid.
  <Side Effects>
    The geniuser password gets changed to the new value (in the db).
  <Returns>
    None
  """
  assert_geniuser(geniuser)
  #Determines if the new password is strong enough.  The frontend should already
  #check for this but we validate again here just in case.
  validations.validate_password(new_password)
  
  # Lock the user.
  lockserver_handle = lockserver.create_lockserver_handle()
  lockserver.lock_user(lockserver_handle, geniuser.username)
  try:
    # Make sure the user still exists now that we hold the lock. Also makes
    # sure that we see any changes made to the user before we obtained the lock.
    # We don't use the user object we retrieve because we want the
    # object passed in to the function to reflect changes we make to the object.
    try:
      maindb.get_user(geniuser.username)
    except DoesNotExistError:
      raise InternalError(traceback.format_exc())

    maindb.set_user_password(geniuser, new_password)
  finally:
    # Unlock the user.
    lockserver.unlock_user(lockserver_handle, geniuser.username)
    lockserver.destroy_lockserver_handle(lockserver_handle)





@log_function_call
def regenerate_api_key(geniuser):
  """
  <Purpose>
    Regenerates the user's API key.
  <Arguments>
    geniuser
      A GeniUser object of the user whose api key is to be regenerated.
  <Exceptions>
    None
  <Side Effects>
    The API key for the user is updated in the database.
  <Returns>
    The new API key.
  """
  assert_geniuser(geniuser)
  
  # Lock the user.
  lockserver_handle = lockserver.create_lockserver_handle()
  lockserver.lock_user(lockserver_handle, geniuser.username)
  try:
    # Make sure the user still exists now that we hold the lock. Also makes
    # sure that we see any changes made to the user before we obtained the lock.
    # We don't use the user object we retrieve because we want the
    # object passed in to the function to reflect changes we make to the object.
    try:
      maindb.get_user(geniuser.username)
    except DoesNotExistError:
      raise InternalError(traceback.format_exc())
    
    return maindb.regenerate_api_key(geniuser)
    
  finally:
    # Unlock the user.
    lockserver.unlock_user(lockserver_handle, geniuser.username)
    lockserver.destroy_lockserver_handle(lockserver_handle) 



  

@log_function_call
def delete_private_key(geniuser):
  """
  <Purpose>
    Deletes the private key of the specified user.
  <Arguments>
    geniuser
      A GeniUser object of the user whose private key is to be deleted.
  <Exceptions>
    None
  <Side Effects>
    The private key belonging to the user is deleted if it exists, otherwise
    the user account is not modified.
  <Returns>
    None
  """
  assert_geniuser(geniuser)
  
  # Lock the user.
  lockserver_handle = lockserver.create_lockserver_handle()
  lockserver.lock_user(lockserver_handle, geniuser.username)
  try:
    # Make sure the user still exists now that we hold the lock. Also makes
    # sure that we see any changes made to the user before we obtained the lock.
    # We don't use the user object we retrieve because we want the
    # object passed in to the function to reflect the deletion of the key.
    # That is, we want the object passed in to have the user_privkey be None
    # when this function returns.
    try:
      maindb.get_user(geniuser.username)
    except DoesNotExistError:
      raise InternalError(traceback.format_exc())
    
    maindb.delete_user_private_key(geniuser)
    
  finally:
    # Unlock the user.
    lockserver.unlock_user(lockserver_handle, geniuser.username)
    lockserver.destroy_lockserver_handle(lockserver_handle)





@log_function_call_without_return
def get_private_key(geniuser):
  """
  <Purpose>
    Gets the private key of the specified user.
  <Arguments>
    geniuser
      A GeniUser object of the user whose private key is to be retrieved.
  <Exceptions>
    None
  <Side Effects>
    None
  <Returns>
    The string containing the user's private key or None if the user's private
    key is not stored by us (e.g. that is, either we never had it or the user
    deleted it).
  """
  assert_geniuser(geniuser)
  
  return geniuser.user_privkey





@log_function_call
def get_donations(geniuser):
  """
  <Purpose>
    Gets a list of donations made by a specific user.
  <Arguments>
    geniuser
      The GeniUser object who is the donor of the donations.
  <Exceptions>
    None
  <Side Effects>
    None
  <Returns>
    A list of the donations made by geniuser.
  """
  assert_geniuser(geniuser)
  
  # This is read-only, so not locking the user.
  return maindb.get_donations_by_user(geniuser)





@log_function_call
def get_acquired_vessels(geniuser):
  """
  <Purpose>
    Gets a list of vessels that have been acquired by the user.
  <Arguments>
    user
      A GeniUser object of the user who is assigned to the vessels.
  <Exceptions>
    None
  <Side Effects>
    None
  <Returns>
    A list of Vessel objects for the vessels that have been acquired by the
    user.
  """
  assert_geniuser(geniuser)
  
  # This is read-only, so not locking the user.
  return maindb.get_acquired_vessels(geniuser)





# @log_action is a decorator that records details of vessel-affecting
# operations in the database. This decorator should be kept in mind whenever
# the arguments or return value to this function are changed.
@log_action
@log_function_call
def acquire_vessels(geniuser, vesselcount, vesseltype):
  """
  <Purpose>
    Acquire unused vessels of a given type for a user. For information on how
    the specified vesseltype affects which vessels will be considered
    to satisfy the request, see the type-specific functions that are called
    by this function.
  <Arguments>
    geniuser
      The GeniUser which will be assigned the vessels.
    vesselcount
      The number of vessels to acquire (a positive integer).
    vesseltype
      The type of vessels to acquire. One of either 'lan', 'wan', 'nat', or 'rand'.
  <Exceptions>
    UnableToAcquireResourcesError
      If not able to acquire the requested vessels (in this case, no vessels
      will be acquired).
    InsufficientUserResourcesError
      The user does not have enough vessel credits to acquire the number of
      vessels requested.
  <Side Effects>
    A total of 'vesselcount' previously-unassigned vessels of the specified
    vesseltype have been acquired by the user.
  <Returns>
    A list of the vessels as a result of this function call.
  """
  assert_geniuser(geniuser)
  assert_positive_int(vesselcount)
  assert_str(vesseltype)

  # Lock the user.
  lockserver_handle = lockserver.create_lockserver_handle()
  lockserver.lock_user(lockserver_handle, geniuser.username)
  
  try:
    # Make sure the user still exists now that we hold the lock. Also makes
    # sure that we see any changes made to the user before we obtained the lock.
    try:
      geniuser = maindb.get_user(geniuser.username)
    except DoesNotExistError:
      raise InternalError(traceback.format_exc())
    
    # Ensure the user is allowed to acquire these resources. This call will
    # raise an InsufficientUserResourcesError if the additional vessels would
    # cause the user to be over their limit.
    maindb.require_user_can_acquire_resources(geniuser, vesselcount)
    
    if vesseltype == 'wan':
      acquired_list = vessels.acquire_wan_vessels(lockserver_handle, geniuser, vesselcount)
    elif vesseltype == 'lan':
      acquired_list = vessels.acquire_lan_vessels(lockserver_handle, geniuser, vesselcount)
    elif vesseltype == 'nat':
      acquired_list = vessels.acquire_nat_vessels(lockserver_handle, geniuser, vesselcount)
    elif vesseltype == 'rand':
      acquired_list = vessels.acquire_rand_vessels(lockserver_handle, geniuser, vesselcount)
    else:
      raise ProgrammerError("Vessel type '%s' is not a valid type" % vesseltype)
    
    return acquired_list
    
  finally:
    # Unlock the user.
    lockserver.unlock_user(lockserver_handle, geniuser.username)
    lockserver.destroy_lockserver_handle(lockserver_handle)





# @log_action is a decorator that records details of vessel-affecting
# operations in the database. This decorator should be kept in mind whenever
# the arguments or return value to this function are changed.
@log_action
@log_function_call
def acquire_specific_vessels(geniuser, vessel_list):
  """
  <Purpose>
    Attempt to acquire specific vessels for a user.
  <Arguments>
    geniuser
      The GeniUser which will be assigned the vessels.
    vessel_list
      A list of vessels to be acquired for the user.
  <Exceptions>
    InsufficientUserResourcesError
      The user does not have enough vessel credits to acquire the number of
      vessels requested.
    InvalidRequestError
      If the list of vessels is empty.
  <Side Effects>
    Zero or more of the vessels in vessel_list have been acquired by the user.
  <Returns>
    A list of the vessels acquired as a result of this function call. The
    length of this list may be less than the length of vessel_list if one or
    more of the vessels in vessel_list could not be acquired.
  """
  assert_geniuser(geniuser)
  assert_list(vessel_list)
  for vessel in vessel_list:
    assert_vessel(vessel)

  if not vessel_list:
    raise InvalidRequestError("The list of vessels cannot be empty.")

  # Lock the user.
  lockserver_handle = lockserver.create_lockserver_handle()
  lockserver.lock_user(lockserver_handle, geniuser.username)
  
  try:
    # Make sure the user still exists now that we hold the lock. Also makes
    # sure that we see any changes made to the user before we obtained the lock.
    try:
      geniuser = maindb.get_user(geniuser.username)
    except DoesNotExistError:
      raise InternalError(traceback.format_exc())
    
    # Ensure the user is allowed to acquire these resources. This call will
    # raise an InsufficientUserResourcesError if the additional vessels would
    # cause the user to be over their limit.
    maindb.require_user_can_acquire_resources(geniuser, len(vessel_list))
    
    return vessels.acquire_specific_vessels_best_effort(lockserver_handle, geniuser, vessel_list)
    
  finally:
    # Unlock the user.
    lockserver.unlock_user(lockserver_handle, geniuser.username)
    lockserver.destroy_lockserver_handle(lockserver_handle)





# @log_action is a decorator that records details of vessel-affecting
# operations in the database. This decorator should be kept in mind whenever
# the arguments or return value to this function are changed.
@log_action
@log_function_call
def release_vessels(geniuser, vessel_list):
  """
  <Purpose>
    Remove a user from a vessel that is assigned to the user.
  <Arguments>
    geniuser
      The GeniUser who is to be removed from the vessel.
    vessel_list
      A list of vessels the user is to be removed from.
  <Exceptions>
    InvalidRequestError
      If any of the vessels in the vessel_list are not currently acquired by
      geniuser or if the list of vessels is empty.
  <Side Effects>
    The vessel is no longer assigned to the user. If this was the last user
    assigned to the vessel, the vessel is freed.
  <Returns>
    None
  """
  assert_geniuser(geniuser)
  assert_list(vessel_list)
  for vessel in vessel_list:
    assert_vessel(vessel)

  if not vessel_list:
    raise InvalidRequestError("The list of vessels cannot be empty.")

  # Lock the user.
  lockserver_handle = lockserver.create_lockserver_handle()
  lockserver.lock_user(lockserver_handle, geniuser.username)
  
  try:
    # Make sure the user still exists now that we hold the lock. Also makes
    # sure that we see any changes made to the user before we obtained the lock.
    try:
      geniuser = maindb.get_user(geniuser.username)
    except DoesNotExistError:
      raise InternalError(traceback.format_exc())
    
    vessels.release_vessels(lockserver_handle, geniuser, vessel_list)

  finally:
    # Unlock the user.
    lockserver.unlock_user(lockserver_handle, geniuser.username)
    lockserver.destroy_lockserver_handle(lockserver_handle)





@log_function_call
def release_all_vessels(geniuser):
  """
  <Purpose>
    Release all vessels that have been acquired by the user.
  <Arguments>
    geniuser
      The GeniUser who is to have their vessels released.
  <Exceptions>
    None
  <Side Effects>
    All of the user's acquired vessels at the time of this function call
    have been released. This function does not guarantee that the user has
    no acquired vessels at the time that it returns, though.
  <Returns>
    None
  """
  # We get the list of vessels without holding a lock. An acquisition
  # request or other request could sneak in after we get the vessel list but
  # before we hold the lock. I'm not going to worry about this because it won't
  # lead to data integrity. At worst the user could get an error about not
  # being able to release all vessels.

  # Get a list of all vessels acquired by the user.
  vessel_list = maindb.get_acquired_vessels(geniuser)
  
  if not vessel_list:
    raise InvalidRequestError("You have no vessels and so nothing to release.")
  
  release_vessels(geniuser, vessel_list)
  




# @log_action is a decorator that records details of vessel-affecting
# operations in the database. This decorator should be kept in mind whenever
# the arguments or return value to this function are changed.
@log_action
@log_function_call
def renew_vessels(geniuser, vessel_list):
  """
  <Purpose>
    Extend the expiration dates of vessels acquired by a user.
  <Arguments>
    geniuser
      The GeniUser whose vessels are to be renewed.
    vessel_list
      A list of vessels to be renewed.
  <Exceptions>
    InvalidRequestError
      If any of the vessels in the vessel_list are not currently acquired by
      geniuser or if the list of vessels is empty.
    InsufficientUserResourcesError
      If the user is currently over their limit of acquired resources.
  <Side Effects>
    The vessels are renewed to the maximum time vessels can be acquired for,
    regardless of their previous individual expiration times.
  <Returns>
    None
  """
  assert_geniuser(geniuser)
  assert_list(vessel_list)
  for vessel in vessel_list:
    assert_vessel(vessel)

  if not vessel_list:
    raise InvalidRequestError("The list of vessels cannot be empty.")

  # Lock the user.
  lockserver_handle = lockserver.create_lockserver_handle()
  lockserver.lock_user(lockserver_handle, geniuser.username)
  
  try:
    # Make sure the user still exists now that we hold the lock. Also makes
    # sure that we see any changes made to the user before we obtained the lock.
    try:
      geniuser = maindb.get_user(geniuser.username)
    except DoesNotExistError:
      raise InternalError(traceback.format_exc())
    
    # Ensure the user is not over their limit of acquired vessels due to
    # donations of theirs having gone offline. This call will raise an
    # InsufficientUserResourcesError if the user is currently over their
    # limit.
    vesselcount = 0
    maindb.require_user_can_acquire_resources(geniuser, vesselcount)
    
    # The vessels.renew_vessels function is responsible for ensuring that the
    # vessels belong to this user. We let the other function do the check
    # because we want to hold locks on the vessels' nodes before checking.
    vessels.renew_vessels(lockserver_handle, geniuser, vessel_list)

  finally:
    # Unlock the user.
    lockserver.unlock_user(lockserver_handle, geniuser.username)
    lockserver.destroy_lockserver_handle(lockserver_handle)





@log_function_call
def renew_all_vessels(geniuser):
  """
  <Purpose>
    Extend the expiration dates of vessels acquired by a user.
  <Arguments>
    geniuser
      The GeniUser whose vessels are to be renewed.
  <Exceptions>
    InvalidRequestError
      If the user has no acquired vessels.
    InsufficientUserResourcesError
      If the user is currently over their limit of acquired resources.
  <Side Effects>
    All vessels acquired by the user at the time of this function all are
    renewed to the maximum time vessels can be acquired for, regardless of
    their previous individual expiration times.
  <Returns>
    None
  """
  # We get the list of vessels without holding a lock. An acquisition
  # request or other request could sneak in after we get the vessel list but
  # before we hold the lock. I'm not going to worry about this because it won't
  # lead to data integrity. At worst the user could get an error about not
  # being able to renew all vessels.
    
  # Get a list of all vessels acquired by the user.
  vessel_list = maindb.get_acquired_vessels(geniuser)
  
  if not vessel_list:
    raise InvalidRequestError("You have no vessels and so nothing to renew.")
  
  renew_vessels(geniuser, vessel_list)





# Not logging the function call for now.
def get_vessel_list(vesselhandle_list):
  """
  <Purpose>
    Convert a list of vesselhandles into a list of Vessel objects.
  <Arguments>
    vesselhandle_list
      A list of strings where each string is a vesselhandle of the format
      "nodeid:vesselname"
  <Exceptions>
    DoesNotExistError
      If a specified vessel does not exist.
    InvalidRequestError
      If any vesselhandle in the list is not in the correct format.
  <Side Effects>
    None
  <Returns>
    A list of Vessel objects.
  """
  assert_list_of_str(vesselhandle_list)
  
  vessel_list = []
  
  for vesselhandle in vesselhandle_list:
    if len((vesselhandle.split(":"))) != 2:
      raise InvalidRequestError("Invalid vesselhandle: " + vesselhandle)
    
    (nodeid, vesselname) = vesselhandle.split(":")
    # Raises DoesNotExistError if there is no such node/vessel.
    vessel = maindb.get_vessel(nodeid, vesselname)
    vessel_list.append(vessel)
    
  return vessel_list


  


# Not logging the function call for now.
def get_vessel_infodict_list(vessel_list):
  """
  <Purpose>
    Convert a list of Vessel objects into a list of vessel infodicts.
    An "infodict" is a dictionary of vessel information that contains data
    which is safe for public display.
    
    This function needs to return lists of dictionaries with a minimum of the
    following, according to https://seattle.cs.washington.edu/wiki/SeattleGeniAPI:
      {'node_ip':node_ip, 'node_port':node_port, 'vessel_id':vessel_id, 
      'node_id':node_id, 'handle':handle}
  <Arguments>
    vessel_list
      A list of Vessel objects.
  <Exceptions>
    None
  <Side Effects>
    None
  <Returns>
    A list of vessel infodicts.
  """
  infodict_list = []
  
  for vessel in vessel_list:
    vessel_info = {}
    
    vessel_info["node_id"] = maindb.get_node_identifier_from_vessel(vessel)
    node = maindb.get_node(vessel_info["node_id"])
    
    vessel_info["node_ip"] = node.last_known_ip
    vessel_info["node_port"] = node.last_known_port
    vessel_info["vessel_id"] = vessel.name
    
    vessel_info["handle"] = vessel_info["node_id"] + ":" + vessel.name
    
    vessel_info["is_active"] = node.is_active
    
    expires_in_timedelta = vessel.date_expires - datetime.datetime.now()
    # The timedelta object stores information in two parts: days and seconds.
    vessel_info["expires_in_seconds"] = (expires_in_timedelta.days * 3600 * 24) + expires_in_timedelta.seconds
      
    infodict_list.append(vessel_info)
    
  return infodict_list





def get_total_vessel_credits(geniuser):
  """
  <Purpose>
    Determine the total number of vessels the user is allowed to acquire,
    regardless of how many they have already acquired.
  <Arguments>
    geniuser
      The GeniUser whose total vessel credit count is wanted.
  <Exceptions>
    None
  <Side Effects>
    None
  <Returns>
    The total number of vessels the user is allowed to acquire, regardless
    of how many they currently have acquired.
  """
  assert_geniuser(geniuser)
  
  return maindb.get_user_total_vessel_credits(geniuser)


def get_free_vessel_credits_amount(geniuser):
  """
  <Purpose>
    Determine the total number of free vessel credits the user is given.
  <Arguments>
    geniuser
      The GeniUser whose free vessel credits is wanted
  <Exceptions>
    None
  <Side Effects>
    None
  <Returns>
    The total number of vessels the user is given for free.
  """
  assert_geniuser(geniuser)

  return maindb.get_user_free_vessel_credits(geniuser)


def get_available_vessel_credits(geniuser):
  """
  <Purpose>
    Determine the number vessels the user is allowed to acquire at the moment
    (that is, the total vessel credits they have minus the number of vessels
    they have acquired).
  <Arguments>
    geniuser
      The GeniUser whose available vessel credit count is wanted.
  <Exceptions>
    None
  <Side Effects>
    None
  <Returns>
    The maximum number of vessels the user is allowed to acquire at this
    moment without exceeding the total number they are allowed to acquire.
  """
  assert_geniuser(geniuser)
  
  max_allowed_vessels = maindb.get_user_total_vessel_credits(geniuser)
  acquired_vessel_count = len(maindb.get_acquired_vessels(geniuser))
  
  if acquired_vessel_count >= max_allowed_vessels:
    return 0
  else:
    return max_allowed_vessels - acquired_vessel_count



"""
<Program>
  decorators.py

<Started>
  6 July 2009

<Author>
  Justin Samuel

<Purpose>
  These define the decorators used in seattlegeni code. Decorators are something
  we try to avoid using, so they should only be used if absolutely necessary.
  Currently we only use them for logging function calls (such as the public
  api functions).

  The simple_decorator approach here is borrowed from:
  http://wiki.python.org/moin/PythonDecoratorLibrary

  The general idea is that we have one simple_decorator that does magic python
  stuff (_simple_decorator), and we write our actual decorators that are more
  sane and are themselves decorated with the _simple_decorator.
"""

import datetime

from seattlegeni.common.util import log

from django.http import HttpRequest


def _simple_decorator(decorator):
  """
  This is not for use outside of this module.
  
  This decorator can be used to turn simple functions into well-behaved
  decorators, so long as the decorators are fairly simple. If a decorator
  expects a function and returns a function (no descriptors), and if it doesn't
  modify function attributes or docstring, then it is eligible to use this.
  Simply apply @simple_decorator to your decorator and it will automatically
  preserve the docstring and function attributes of functions to which it is
  applied.
  """
  def new_decorator(f):
    g = decorator(f)
    g.__name__ = f.__name__
    g.__doc__ = f.__doc__
    g.__dict__.update(f.__dict__)
    return g
  # Now a few lines needed to make simple_decorator itself
  # be a well-behaved decorator.
  new_decorator.__name__ = decorator.__name__
  new_decorator.__doc__ = decorator.__doc__
  new_decorator.__dict__.update(decorator.__dict__)
  return new_decorator





def _get_timedelta_str(starttime):
  return str(datetime.datetime.now() - starttime)





@_simple_decorator
def log_function_call(func):
  """
  <Purpose>
    Logs when the function is called, along with the arguments, and logs when
    the function returns, along with the return value. Will also log any
    exception that is raised.
    
    Be careful when using this to log functions that take sensitive values
    (e.g. passwords) as arguments or that return sensitive values (e.g.
    private keys).
  """
  
  # The name "do_logging_func_call" is never seen anywhere but here.
  def do_logging_func_call(*args, **kwargs):
    _log_call_info(func, args, kwargs)
    
    starttime = datetime.datetime.now()
    
    try:
      result = func(*args, **kwargs)
      log.debug('Returning from %s (module %s) (time %s): %s' % (func.__name__, func.__module__, _get_timedelta_str(starttime), str(result)))
      return result
    
    except Exception, e:
      log.debug('Exception from %s (module %s): %s %s' % (func.__name__, func.__module__, type(e), str(e)))
      raise
    
  return do_logging_func_call





@_simple_decorator
def log_function_call_without_return(func):
  """
  <Purpose>
    Logs when the function is called, along with the arguments, and logs when
    the function returns, but doesn't log the return value. Will also log any
    exception that is raised.
    
    Be careful when using this to log functions that take sensitive values
    (e.g. passwords) as arguments.
  """
  
  # The name "do_logging_func_call" is never seen anywhere but here.
  def do_logging_func_call(*args, **kwargs):
    _log_call_info(func, args, kwargs)
    
    starttime = datetime.datetime.now()
    
    try:
      result = func(*args, **kwargs)
      log.debug('Returning from %s (module %s) (time %s): [Not logging return value]' % (func.__name__, func.__module__, _get_timedelta_str(starttime)))
      return result
    
    except Exception, e:
      log.debug('Exception from %s (module %s): %s %s' % (func.__name__, func.__module__, type(e), str(e)))
      raise
  
  return do_logging_func_call





@_simple_decorator
def log_function_call_without_arguments(func):
  """
  <Purpose>
    Logs when the function is called, without the arguments, and logs when
    the function returns, including the return value. Will also log any
    exception that is raised.
    
    Be careful when using this to log functions that return sensitive values
    (e.g. private keys).
  """
  
  # The name "do_logging_func_call" is never seen anywhere but here.
  def do_logging_func_call(*args, **kwargs):
    log.debug('Calling: %s (module %s), args: [Not logging], kwargs: [Not logging].' %
             (func.__name__, func.__module__))
    
    starttime = datetime.datetime.now()
    
    try:
      result = func(*args, **kwargs)
      log.debug('Returning from %s (module %s) (time %s): %s' % (func.__name__, func.__module__, _get_timedelta_str(starttime), str(result)))
      return result
    
    except Exception, e:
      log.debug('Exception from %s (module %s): %s %s' % (func.__name__, func.__module__, type(e), str(e)))
      raise
  
  return do_logging_func_call





@_simple_decorator
def log_function_call_and_only_first_argument(func):
  """
  <Purpose>
    Logs when the function is called, with only the first, and logs when
    the function returns, including the return value. Will also log any
    exception that is raised.
    
    Be careful when using this to log functions that return sensitive values
    (e.g. private keys).
    
    The reason this decorator exists is that there are a handful of functions
    that take sensitive data as arguments (like passwords) but they are
    not the first argument, and logging the first argument could be useful.
    This could probably be accomplished by making a decorator that itself
    took arguments about which arguments to log, but that crosses well over
    the line of maintainability by people who didn't write the initial code.
  """
  
  # The name "do_logging_func_call" is never seen anywhere but here.
  def do_logging_func_call(*args, **kwargs):
    log.debug('Calling: %s (module %s), 1st arg: %s, other args: [Not logging].' % 
             (func.__name__, func.__module__, str(_get_cleaned_args(args)[0])))
    
    starttime = datetime.datetime.now()
    
    try:
      result = func(*args, **kwargs)
      log.debug('Returning from %s (module %s) (time %s): %s' % (func.__name__, func.__module__, _get_timedelta_str(starttime), str(result)))
      return result
    
    except Exception, e:
      log.debug('Exception from %s (module %s): %s %s' % (func.__name__, func.__module__, type(e), str(e)))
      raise
  
  return do_logging_func_call





@_simple_decorator
def log_function_call_without_first_argument(func):
  """
  <Purpose>
    Logs the function called without the first argument (unless it's a kwarg),
    and logs when the function returns including the return value. Will also log
    any exception that is raised.
    
    Be careful when using this to log functions that return sensitive values
    (e.g. private keys).
    
    The reason this decorator exists is that there are functions that take
    a sensitive value (such as the backend authcode) as the first argument,
    and we don't want that ending up in the logs.
  """
  
  # The name "do_logging_func_call" is never seen anywhere but here.
  def do_logging_func_call(*args, **kwargs):
    log.debug('Calling: %s (module %s), 1st arg: [Not logging], other args: %s, kwargs: %s.' % 
             (func.__name__, func.__module__, str(_get_cleaned_args(args)[1:]), str(_get_cleaned_args(kwargs))))
    
    starttime = datetime.datetime.now()
    
    try:
      result = func(*args, **kwargs)
      log.debug('Returning from %s (module %s) (time %s): %s' % (func.__name__, func.__module__, _get_timedelta_str(starttime), str(result)))
      return result
    
    except Exception, e:
      log.debug('Exception from %s (module %s): %s %s' % (func.__name__, func.__module__, type(e), str(e)))
      raise
  
  return do_logging_func_call





def _log_call_info(func, args, kwargs):
  # This is a separate function because I didn't want to repeat it the code in
  # both log_function_call and log_function_call_without_return.
  
  # TODO: clean up this line
  log.debug('Calling: %s (module %s), args: %s, kwargs: %s.' %
           (func.__name__, func.__module__, str(_get_cleaned_args(args)), str(_get_cleaned_args(kwargs))))




def _get_cleaned_args(args):
  
  cleanedargslist = []

  for item in args:
    if isinstance(item, HttpRequest):
      cleanedargslist.append("<HttpRequest>")
    else:
      cleanedargslist.append(item)
    
  return tuple(cleanedargslist)
      
      
      
      
      
def _get_cleaned_kwargs(kwargs):
  
  cleanedkwargs = {}

  for key in kwargs:
    if isinstance(kwargs[key], HttpRequest):
      cleanedkwargs[key] = "<HttpRequest>"
    elif key == "password":
      cleanedkwargs[key] = "***"
    else:
      cleanedkwargs[key] = kwargs[key]
    
  return cleanedkwargs
  



"""
<Program>
  test_register.py

<Started>
  8/26/2009

<Author>
  Jason Chen
  jchen@cs.washington.edu

<Purpose>
  Tests the registration function of the HTML frontend view.

<Notes>
  We make use of the Django test client, which emulates a webclient and returns what django
  considers the final rendered result (in HTML). This allows us to test purely the view functions.
  Apart from testing the validity/sanity of the view functions, we also mock-out all calls into the
  interface, and return test values to the view functions. This way, we can ensure that the views behave
  properly in normal (and abnormal) interactions with the interface.
"""

# We import the testlib FIRST, as the test db settings 
# need to be set before we import anything else.
from seattlegeni.tests import testlib

import StringIO
import os

from seattlegeni.common.exceptions import *
from seattlegeni.common.util import validations
from seattlegeni.website.control import interface
from seattlegeni.website.control import models
from seattlegeni.website.html import forms

from django.contrib.auth.models import User as DjangoUser

# The django test client emulates a webclient, and returns what django
# considers the final rendered result (in HTML). This allows to test purely the view
# functions. 
from django.test.client import Client


# Declare our mock functions
def mock_register_user(username, password, email, affiliation, pubkey=None):
  testuser = DjangoUser()
  testuser.username = "test_user"
  return testuser

def mock_register_user_throws_ValidationError(username, password, email, affiliation, pubkey=None):
  raise ValidationError

def mock_get_logged_in_user(request):
  raise DoesNotExistError

# set up test data
c = Client()
good_data = {'username': 'tester', 
             'password1': '12345678',
             'password2': '12345678',  
             'email': 'test@test.com',  
             'affiliation': 'test affil', 
             'gen_upload_choice':'1'}

test_data = {}

def main():
  
  # Setup test environment
  testlib.setup_test_environment()
  testlib.setup_test_db()
  
  try:
    # mock out interface calls
    interface.register_user = mock_register_user
    interface.get_logged_in_user = mock_get_logged_in_user
    
    # run tests
    test_post_blank_form()
    test_short_username()
    test_long_username()
    test_invalid_username()
    test_short_password()
    test_nonmatching_password()
    test_matching_username_password()
    test_short_affil()
    test_long_affil()
    test_invalid_email()
    test_key_upload_with_no_file()
    test_key_upload_with_oversized_file()
    test_key_upload_with_bad_format_key()
    test_key_upload_with_valid_key()
    test_normal()
    
    print "All tests passed."
    
  finally:
    testlib.teardown_test_db()
    testlib.teardown_test_environment()

  

def test_post_blank_form():
  """
  <Purpose>
    Test for posting blank form
  
  <Note>
    Checking for the existance of (p class=\"warning\") in response.content is
    sufficient in detecting whether the view caught an error; since the page
    will only ever show that html element in the event of an error. And, this 
    is all we care about: that the page recognized that bad data was passed in, 
    and an error was raised.
  """
  response = c.post('/html/register', {'username': '', 
                                       'password1':'', 
                                       'password2':'', 
                                       'email':'', 
                                       'affiliation':'', 
                                       'gen_upload_choice':'1'}, follow=True)
  
  assert(response.status_code == 200)
  assert("p class=\"warning\"" in response.content)



def test_short_username():
  """
  <Purpose>
    Test for username too short
  """  
  shortuser = ""
  for n in range(0, validations.USERNAME_MIN_LENGTH - 1):
    shortuser += "a"
  
  test_data = good_data.copy()
  test_data['username'] = shortuser
  
  response = c.post('/html/register', test_data, follow=True)
  assert(response.status_code == 200)
  assert("p class=\"warning\"" in response.content)



def test_long_username():
  """
  <Purpose>
    Test for username too long
  """  
  longuser = ""
  for n in range(0, validations.USERNAME_MAX_LENGTH + 1):
    longuser += "a"
  
  test_data = good_data.copy()
  test_data['username'] = longuser
  
  response = c.post('/html/register', test_data, follow=True)
  assert(response.status_code == 200)
  assert("p class=\"warning\"" in response.content)



def test_invalid_username():
  """
  <Purpose>
    Test for invalid username
  """  
  test_data = good_data.copy()
  test_data['username'] = 'tester!!!'
  
  response = c.post('/html/register', test_data, follow=True)
  assert(response.status_code == 200)
  assert("p class=\"warning\"" in response.content)



def test_short_password():
  """
  <Purpose>
    Test for password too short
  """  
  test_data = good_data.copy()
  test_data['password1'] = '12345'
  
  response = c.post('/html/register', test_data, follow=True)
  assert(response.status_code == 200)
  assert("p class=\"warning\"" in response.content)



def test_nonmatching_password():
  """
  <Purpose>
    Test for non-matching passwords
  """  
  test_data = good_data.copy()
  test_data['password2'] = '87654321'
  
  response = c.post('/html/register', test_data, follow=True)
  assert(response.status_code == 200)
  assert("p class=\"warning\"" in response.content)



def test_matching_username_password():
  """
  <Purpose>
    Test for same username and password
  """  
  test_data = good_data.copy()
  test_data['username'] = 'tester'
  test_data['password1'] = 'tester'
  test_data['password2'] = 'tester'
  
  response = c.post('/html/register', test_data, follow=True)
  assert(response.status_code == 200)
  assert("p class=\"warning\"" in response.content)



def test_short_affil():
  """
  <Purpose>
    Test for affiliation too short
  """  
  shortaffil = ""
  for n in range(0, validations.AFFILIATION_MIN_LENGTH - 1):
    shortaffil += "a"
  
  test_data = good_data.copy()
  test_data['affiliation'] = shortaffil
  
  response = c.post('/html/register', test_data, follow=True)
  assert(response.status_code == 200)
  assert("p class=\"warning\"" in response.content)



def test_long_affil():
  """
  <Purpose>
    Test for affiliation too long
  """  
  longaffil = ""
  for n in range(0, validations.AFFILIATION_MAX_LENGTH + 1):
    longaffil += "a"
  
  test_data = good_data.copy()
  test_data['affiliation'] = longaffil
  
  response = c.post('/html/register', test_data, follow=True)
  assert(response.status_code == 200)
  assert("p class=\"warning\"" in response.content)



def test_invalid_email():
  """
  <Purpose>
    Test for invalid e-mail
  """  
  test_data = good_data.copy()
  test_data['email'] = 'invalid@email'
  
  response = c.post('/html/register', test_data, follow=True)
  assert(response.status_code == 200)
  assert("p class=\"warning\"" in response.content)



def test_key_upload_with_no_file():
  """
  <Purpose>
    Test key upload with no file
  """  
  test_data = good_data.copy()
  test_data['gen_upload_choice'] = '2'
  
  response = c.post('/html/register', test_data, follow=True)
  assert(response.status_code == 200)
  assert("p class=\"warning\"" in response.content)



def test_key_upload_with_oversized_file():
  """
  <Purpose>
    Test key upload with oversized file
  """  
  test_data = good_data.copy()
  test_data['gen_upload_choice'] = '2'
  
  # set up a temporary 'key' file object
  data = ''
  for n in range(0, forms.MAX_PUBKEY_UPLOAD_SIZE + 1):
    data += '0'
  
  fh = open(os.path.join(os.getcwd(), 'bigkey'), 'w')
  fh.write(data)
  fh.close()
  fh = open('bigkey')
  test_data['pubkey'] = fh
  
  response = c.post('/html/register', test_data, follow=True)
  fh.close()
  os.remove(os.path.join(os.getcwd(), 'bigkey'))
  
  assert(response.status_code == 200)
  assert("p class=\"warning\"" in response.content)



def test_key_upload_with_bad_format_key():
  """
  <Purpose>
    Test key upload with badly formatted key
  """  
  test_data = good_data.copy()
  test_data['gen_upload_choice'] = '2'
  
  data = 'this is a fake key'
  fh = open(os.path.join(os.getcwd(), 'fakekey'), 'w')
  fh.write(data)
  fh.close()
  fh = open('fakekey')
  test_data['pubkey'] = fh
  
  response = c.post('/html/register', test_data, follow=True)
  fh.close()
  os.remove(os.path.join(os.getcwd(), 'fakekey'))
  
  assert(response.status_code == 200)
  assert("p class=\"warning\"" in response.content)



def test_key_upload_with_valid_key():
  """
  <Purpose>
    Test key upload with valid key
  """  
  test_data = good_data.copy()
  test_data['gen_upload_choice'] = '2'
  
  data = '65537 1255529510320394315881981094520474315188224118590997402913761676258431241305254181126132200564438402053537493727356349330161335739149834523475129922460441305694528141374947142391183883986671526830986555904385942930184161960677388507'
  fh = open(os.path.join(os.getcwd(), 'goodkey'), 'w')
  fh.write(data)
  fh.close()
  fh = open('goodkey')
  test_data['pubkey'] = fh
  
  response = c.post('/html/register', test_data, follow=True)
  fh.close()
  os.remove(os.path.join(os.getcwd(), 'goodkey'))
  
  assert(response.status_code == 200)
  assert("successfully registered" in response.content)



def test_interface_throws_ValidationError():
  """
  <Purpose>
    Test behavior if interface throws a ValidationError, 
    even after passing all validation
  """  
  interface.register_user = mock_register_user_throws_ValidationError
  response = c.post('/html/register', good_data, follow=True)
  
  assert(response.status_code == 200)
  assert("p class=\"warning\"" in response.content)



def test_normal():
  """
  <Purpose>
    Test normal behavior
  """  
  interface.register_user = mock_register_user
  response = c.post('/html/register', good_data, follow=True)
  
  # check that the view thinks the user has been registered
  assert(response.status_code == 200)
  assert("successfully registered" in response.content)
  
  # check that the current page is now the login page
  assert(response.template[0].name == "accounts/login.html")


if __name__=="__main__":
  main()


"""
<Program>
  transition_onepercentmanyevents_to_onepercentmanyevents.py

<Purpose>
  The purpose of this program is to update the database about the 
  node, so new things such as seattle version and other stuff are
  reflected in the database

<Started>
  August 13, 2009

<Author>
  Monzur Muhammad
  monzum@cs.washington.edu

<Usage>
  Ensure that seattlegeni and seattle are in the PYTHONPATH.
  Ensure that the database is setup properly and django settings
    are set correctly.

  python transition_onepercentmanyevents_to_onepercentmanyevents.py
"""


from seattlegeni.node_state_transitions import node_transition_lib


def main():
  """
  <Purpose>
    The main function that calls the process_nodes_and_change_state() function
    in the node_transition_lib passing in the process and error functions.

  <Arguments>
    None

  <Exceptions>
    None

  <Side Effects>
    None
  """

  # Mark the node as active.
  mark_node_active = True
  state_function_arg_tuplelist = [
    ("onepercentmanyevents", "onepercentmanyevents", node_transition_lib.update_database, 
     node_transition_lib.noop, mark_node_active, node_transition_lib.update_database_node)]

  sleeptime = 10
  process_name = "onepercentmanyevents_to_onepercentmanyevents"
  parallel_instances = 10

  #call process_nodes_and_change_state() to start the node state transition
  node_transition_lib.process_nodes_and_change_state(state_function_arg_tuplelist, process_name, sleeptime, parallel_instances)





if __name__ == '__main__':
  main()



from django.conf.urls.defaults import *

# Uncomment the next two lines to enable the admin:
# from django.contrib import admin
# admin.autodiscover()
# the format for urlpatterns is
# (regular exp, function, optional dictionary, optional name)
urlpatterns = patterns('seattlegeni.website.html.views',

                       # Previously defined in accounts/urls.py.                       
                       (r'^register$', 'register',{},'register'),
                       (r'^login$', 'login',{},'login'), 
                       (r'^logout$', 'logout',{},'logout'),
                       (r'^accounts_help$', 'accounts_help',{},'accounts_help'),
                       #(r'^simplelogin$', 'simplelogin',{},'simplelogin'), 
                       # OpenID/OAuth error pages
                       (r'^error$', 'error',{}, 'error'),
                       (r'^associate_error$', 'associate_error',{}, 'associate_error'),
                       # OpenID/OAuth auto register page
                       (r'^auto_register$', 'auto_register',{},'auto_register'),
                       # Top level urls and functions:
                       # show the user info page for this user listing the public/private keys, and user information
                       (r'^profile$', 'profile', {}, 'profile'), # was user_info
                       # OpenID/OAuth auto registered users get sent here after creation
                       #(r'^new_auto_register_user$', 'new_auto_register_user', {}, 'new_auto_register_user'), #currently not used
                       # show the used resources page (with all the currently acquired vessels)
                       (r'^myvessels$', 'myvessels', {}, 'myvessels'), # was used_resources
                       # show the help page
                       (r'^help$', 'help', {}, 'help'),
                       # getdonations page (to download installers)
                       (r'^getdonations$', 'getdonations', {}, 'getdonations'),

                       # 'My GENI' page functions:
                       # get new resources (from form)
                       (r'^get_resources$', 'get_resources', {}, 'get_resources'),

                       # delete some specific resource for this user (from form)
                       (r'^del_resource$', 'del_resource', {}, 'del_resource'),
                       # delete all resources for this user (from form)
                       (r'^del_all_resources$', 'del_all_resources', {}, 'del_all_resources'),

                       # renew some specific resource for this user (from form)
                       (r'^renew_resource$', 'renew_resource', {}, 'renew_resource'),
                       # renew all resource for this user (from form)
                       (r'^renew_all_resources$', 'renew_all_resources', {}, 'renew_all_resources'),

                       # Display and allow changing the API key.
                       (r'^api_info$', 'api_info', {}, 'api_info'),

                       # Form to generate or upload a new key.
                       (r'^change_key$', 'change_key', {}, 'change_key'),
                       
                       # Profile page functions:
                       # delete the user's private key from the server (from form)
                       (r'^del_priv$', 'del_priv', {}, 'del_priv'),
                       # download the user's private key (from form)
                       (r'^priv_key$', 'priv_key', {}, 'priv_key'),
                       # download the user's public key (from form)
                       (r'^pub_key$', 'pub_key', {}, 'pub_key'),
                       
#                       (r'^private/$', 'home.views.private'),
#                      # create a new share with another use (from form)
#                      (r'^new_share$', 'new_share', {}, 'new_share'),
#                      # delete an existing share with another user (from form)
#                      (r'^del_share$', 'del_share', {}, 'del_share'),

                       # AJAX functions, called by the 'My GENI' page
                       #(r'^ajax_getcredits$', 'ajax_getcredits', {}, 'ajax_getcredits'),
                       #(r'^ajax_getshares$', 'ajax_getshares', {}, 'ajax_getshares'),
                       #(r'^ajax_editshare$', 'ajax_editshare', {}, 'ajax_editshare'),
                       #(r'^ajax_createshare$', 'ajax_createshare', {}, 'ajax_createshare'),
                       #(r'^ajax_getvessels$', 'ajax_getvessels', {}, 'ajax_getvesseles'),
                      )



"""
Based on:
  http://www.technobabble.dk/2008/apr/02/xml-rpc-dispatching-through-django-test-client/

With the addition of the following to make it work:
  self._use_datetime = False
"""

# We import the testlib FIRST, as the test db settings 
# need to be set before we import anything else.
from seattlegeni.tests import testlib

import cStringIO
import xmlrpclib

from django.test.client import Client

class TestTransport(xmlrpclib.Transport):
  """Handles connections to XML-RPC server through Django test client."""

  def __init__(self, *args, **kwargs):
    self.client = Client()

  def request(self, host, handler, request_body, verbose=0):
    self.verbose = verbose
    self._use_datetime = False
    response = self.client.post(handler,
                                request_body,
                                content_type="text/xml")
    res = cStringIO.StringIO(response.content)
    res.seek(0)
    return self.parse_response(res)



"""
<Program>
  transition_donation_to_canonical.py

<Purpose>
  The purpose of this program is to transition nodes from the
  donation state to the canonical state. The canonical state
  is the state from which we can do anything sensible. It is the
  state that all nodes should return to eventually before going
  to the next state

<Started>
  August 13, 2009

<Author>
  Monzur Muhammad
  monzum@cs.washington.edu

<Usage>
  Ensure that seattlegeni and seattle are in the PYTHONPATH.
  Ensure that the database is setup properly and django settings
    are set correctly.

  python transition_canonical_to_onepercentmanyevents.py
"""


import node_transition_lib





def main():
  """
  <Purpose>
    The main function that calls the process_nodes_and_change_state() function
    in the node_transition_lib passing in the process and error functions.

  <Arguments>
    None

  <Exceptions>
    None

  <Side Effects>
    None
  """

  state_function_arg_tuplelist = [("acceptdonation", "canonical",  node_transition_lib.noop, node_transition_lib.noop, False)]

  sleeptime = 10
  process_name = "donation_to_canonical"
  parallel_instances = 10

  #call process_nodes_and_change_state() to start the node state transition
  node_transition_lib.process_nodes_and_change_state(state_function_arg_tuplelist, process_name, sleeptime, parallel_instances)





if __name__ == '__main__':
  main()



import inspect
import sys
# No real preference for minimock, just the first one I used that did what
# I needed.
from minimock import Mock

def mock_module(module_name):
  """
  Replaces all of the functions of a module with mock versions of the same
  functions that don't do anything. Doesn't modify classes or the methods
  of classes, nor does it change any global data in the modules.
  
  Example usage:
    mock_module('seattlegeni.common.api.lockserver')
  """
  __import__(module_name)
  module = sys.modules[module_name]
  for item_name in dir(module):
    obj = getattr(module, item_name)
    if inspect.isfunction(obj):
      print "Mocking: " + module_name + "." + item_name
      setattr(module, item_name, Mock(obj))



"""
<Program>
  keygen.py

<Started>
  29 June 2009

<Author>
  Justin Samuel

<Purpose>
   This is the API that should be used to generate new public/private key pairs.
"""

import traceback

from seattlegeni.common.exceptions import *

from seattlegeni.common.util import log

from seattlegeni.common.util.decorators import log_function_call_without_return

from seattle.repyportability import *
add_dy_support(locals())

dy_import_module_symbols("rsa.r2py")






# Set to True to obtain keys from the key daemon, false to generate keys
# directly (potentially making some parts of seattlegeni run very slow).
USE_KEYDAEMON = False

KEYDAEMON_HOST = "127.0.0.1"

KEYDAEMON_PORT = "8030"

# If USE_KEYDAEMON is False and so we manually/directly generate keys, this
# is the bit size of the keys we generate.
MANUAL_GENERATION_BITSIZE = 1024





@log_function_call_without_return
def generate_keypair():
  """
  <Purpose>
    Obtain a new (unused) public/private keypair.
  <Arguments>
    None
  <Exceptions>
    None
  <Side Effects>
    Requests a key from the keygen daemon if USE_KEYDAEMON is True. If that
    fails or if USE_KEYDAEMON is False, directly generates a key.
  <Returns>
    A tuple in the format (pubkeystr, privkeystr).
  """
  
  if USE_KEYDAEMON:
    try:
      return _generate_keypair_from_key_daemon()
    except:
      log.critical("Unable to generate key from key daemon, falling back to " + 
                   "manual key generation. This may be very slow. The error " +
                   " from the key daemon was: " + traceback.format_exc())

  return _generate_keypair_directly()





@log_function_call_without_return
def _generate_keypair_directly():
  
  (pubkeydict, privkeydict) = rsa_gen_pubpriv_keys(MANUAL_GENERATION_BITSIZE)

  pubkeystring = rsa_publickey_to_string(pubkeydict)
  privkeystring = rsa_privatekey_to_string(privkeydict)
  
  return (pubkeystring, privkeystring)





@log_function_call_without_return
def _generate_keypair_from_key_daemon():
  
  # TODO: implement obtaining keys from the key daemon.
  raise NotImplementedError




import unittest

import lockserver_daemon as lockserver


class TheTestCase(unittest.TestCase):
  
  def setUp(self):
    # Reset the lockserver's global variables between each test.
    lockserver.init_globals()
    
    
  def testSessionIdThatIsNotAString(self):
    
    session_id = 1
    
    func = lockserver._assert_valid_lockdict
    args = (session_id,)
    self.assertRaises(lockserver.LockserverInvalidRequestError, func, *args)
    
    session_id = []
    
    func = lockserver._assert_valid_lockdict
    args = (session_id,)
    self.assertRaises(lockserver.LockserverInvalidRequestError, func, *args)
    
    session_id = None
    
    func = lockserver._assert_valid_lockdict
    args = (session_id,)
    self.assertRaises(lockserver.LockserverInvalidRequestError, func, *args)
    
    
  def testSessionIdThatIsAnEmptyString(self):
    
    session_id = ''
    
    func = lockserver._assert_valid_lockdict
    args = (session_id,)
    self.assertRaises(lockserver.LockserverInvalidRequestError, func, *args)
    
    
  def testLockdictWithNoLocks(self):
    
    locks = {}
    
    func = lockserver._assert_valid_lockdict
    args = (locks,)
    self.assertRaises(lockserver.LockserverInvalidRequestError, func, *args)
    

  def testLockdictWithUnknownLockType(self):
    
    # 'xxx' is not a valid type of lock.
    locks = {'xxx':['bob']}
    
    func = lockserver._assert_valid_lockdict
    args = (locks,)
    self.assertRaises(lockserver.LockserverInvalidRequestError, func, *args)
    
    
  def testLockdictWithListOfNamesThatIsNotAList(self):
    
    locks = {'user':{}}
    
    func = lockserver._assert_valid_lockdict
    args = (locks,)
    self.assertRaises(lockserver.LockserverInvalidRequestError, func, *args)
    
    locks = {'user':None}
    
    func = lockserver._assert_valid_lockdict
    args = (locks,)
    self.assertRaises(lockserver.LockserverInvalidRequestError, func, *args)
    
    
  def testLockdictWithEmptyListOfLockNames(self):
    
    locks = {'user':[]}
    
    func = lockserver._assert_valid_lockdict
    args = (locks,)
    self.assertRaises(lockserver.LockserverInvalidRequestError, func, *args)
    
    locks = {'node':[]}
    
    func = lockserver._assert_valid_lockdict
    args = (locks,)
    self.assertRaises(lockserver.LockserverInvalidRequestError, func, *args)


  def testLockdictWithEmptyStringInLockNames(self):
    
    locks = {'user':['bob', '']}
    
    func = lockserver._assert_valid_lockdict
    args = (locks,)
    self.assertRaises(lockserver.LockserverInvalidRequestError, func, *args)
    
    locks = {'node':['', '123']}
    
    func = lockserver._assert_valid_lockdict
    args = (locks,)
    self.assertRaises(lockserver.LockserverInvalidRequestError, func, *args)


  def testLockdictWithLockTypesThatAreNotStrings(self):
    
    locks = {123:['bob']}
    
    func = lockserver._assert_valid_lockdict
    args = (locks,)
    self.assertRaises(lockserver.LockserverInvalidRequestError, func, *args)
    

  def testLockdictWithLockNamesThatAreNotStrings(self):
    
    locks = {'user':['bob', None]}
    
    func = lockserver._assert_valid_lockdict
    args = (locks,)
    self.assertRaises(lockserver.LockserverInvalidRequestError, func, *args)
    
    locks = {'node':[123, '123']}
    
    func = lockserver._assert_valid_lockdict
    args = (locks,)
    self.assertRaises(lockserver.LockserverInvalidRequestError, func, *args)
    
    locks = {'node':[[], '123']}
    
    func = lockserver._assert_valid_lockdict
    args = (locks,)
    self.assertRaises(lockserver.LockserverInvalidRequestError, func, *args)
    
    



"""
<Program>
  test_get_resources.py

<Started>
  8/30/2009

<Author>
  Jason Chen
  jchen@cs.washington.edu

<Purpose>
  Tests that the get_resources view function 
  handles normal operation and exceptions correctly.
  
<Notes>
  See test_register.py for an explanation of our usage of the Django test client.
"""

#pragma out


# We import the testlib FIRST, as the test db settings 
# need to be set before we import anything else.
from seattlegeni.tests import testlib

from seattlegeni.common.exceptions import *
from seattlegeni.website.control import interface
from seattlegeni.website.control import models

from django.contrib.auth.models import User as DjangoUser

# The django test client emulates a webclient, and returns what django
# considers the final rendered result (in HTML). This allows to test purely the view
# functions. 
from django.test.client import Client


# Declare our mock functions
def mock_get_logged_in_user(request):
  geniuser = models.GeniUser(username='tester', password='password', email='test@test.com',
                             affiliation='test affil', user_pubkey='user_pubkey',
                             user_privkey='user_privkey', donor_pubkey='donor_pubkey',
                             usable_vessel_port='12345', free_vessel_credits=10)
  return geniuser

def mock_acquire_vessels(geniuser, vesselcount, vesseltype):
  return ['test1', 'test2']

def mock_acquire_vessels_throws_UnableToAcquireResourcesError(geniuser, vesselcount, vesseltype):
  raise UnableToAcquireResourcesError

def mock_acquire_vessels_throws_InsufficientUserResourcesError(geniuser, vesselcount, vesseltype):
  raise InsufficientUserResourcesError

c = Client()
good_data = {'num':5, 'env':'rand'}


def main():
  
  # Setup test environment
  testlib.setup_test_environment()
  testlib.setup_test_db()
  
  try:
    login_test_user()
    
    test_normal()
    test_interface_throws_UnableToAcquireResourcesError()
    test_interface_throws_InsufficientUserResourcesError()
    test_blank_POST_data()
    test_invalid_POST_data_invalid_num()
    test_invalid_POST_data_invalid_env()
    
    print "All tests passed."
    
  finally:
    testlib.teardown_test_db()
    testlib.teardown_test_environment()



def test_normal():
  """
  <Purpose>
    Test normal behavior
  """
  interface.acquire_vessels = mock_acquire_vessels
  response = c.post('/html/get_resources', good_data, follow=True)
  assert(response.status_code == 200)
  assert(response.template[0].name == 'control/myvessels.html')


def test_interface_throws_UnableToAcquireResourcesError():
  """
  <Purpose>
    Test behavior if interface throws UnableToAcquireResourcesError.
  
  <Note>
    Checking only the existance of "action_summary" in the response
    is sufficient in seeing whether the view caught the error, since 
    error messages are shown in that html element. (Of course, so are
    normal messages, but we are clearly having the interface throw an
    exception, so normal messages don't even exist in this context).
  """
  interface.acquire_vessels = mock_acquire_vessels_throws_UnableToAcquireResourcesError
  response = c.post('/html/get_resources', good_data, follow=True)
  
  assert(response.status_code == 200)
  assert(response.template[0].name == 'control/myvessels.html')
  assert("Unable to acquire vessels at this time" in response.content)



def test_interface_throws_InsufficientUserResourcesError():
  """
  <Purpose>
    Test behavior if interface throws InsufficientUserResourcesError.
  """
  interface.acquire_vessels = mock_acquire_vessels_throws_InsufficientUserResourcesError
  response = c.post('/html/get_resources', good_data, follow=True)
  
  assert(response.status_code == 200)
  assert(response.template[0].name == 'control/myvessels.html')
  assert("Unable to acquire" in response.content)
  assert("vessel credit" in response.content)



def test_blank_POST_data():
  """
  <Purpose>
    Test behavior if we submit blank POST data.
  """
  interface.acquire_vessels = mock_acquire_vessels
  response = c.post('/html/get_resources', {}, follow=True)
  
  assert(response.status_code == 200)
  assert(response.template[0].name == 'control/myvessels.html')
  assert("This field is required" in response.content)



def test_invalid_POST_data_invalid_num():
  """
  <Purpose>
    Test behavior if we submit POST data with an invalid 'num' field.
  """
  interface.acquire_vessels = mock_acquire_vessels
  test_data = {'num':-5, 'env':'rand'}
  response = c.post('/html/get_resources', test_data, follow=True)
  
  assert(response.status_code == 200)
  assert(response.template[0].name == 'control/myvessels.html')
  assert("Select a valid choice" in response.content)



def test_invalid_POST_data_invalid_env():
  """
  <Purpose>
    Test behavior if we submit POST data with an invalid 'env' field.
  """
  interface.acquire_vessels = mock_acquire_vessels
  test_data = {'num':5, 'env':'notvalid'}
  response = c.post('/html/get_resources', test_data, follow=True)
  
  assert(response.status_code == 200)
  assert(response.template[0].name == 'control/myvessels.html')
  assert("Select a valid choice" in response.content)



# Creates a test user in the test db, and uses the test client to 'login',
# so all views that expect @login_required will now pass the login check. 
def login_test_user():
  # uses the mock get_logged_in_user function that represents a logged in user
  interface.get_logged_in_user = mock_get_logged_in_user
  
  user = DjangoUser.objects.create_user('tester', 'test@test.com', 'testpassword')
  user.save()
  c.login(username='tester', password='testpassword')



if __name__=="__main__":
  main()